@@ -14,11 +14,14 @@ License: GPL v2.  See License.txt.
 
 *//*******************************************************************/
 
-
+// Store function pointers here when including FFmpeg.h
+#define DEFINE_FFMPEG_POINTERS
 
 #include "Audacity.h"	// for config*.h
 #include "FFmpeg.h"
 #include "AudacityApp.h"
+#include "FileNames.h"
+#include "Internat.h"
 
 #include <wx/file.h>
 
@@ -30,6 +33,8 @@ License: GPL v2.  See License.txt.
    #endif
 #endif
 
+#define UFILE_PROTOCOL "ufile"
+
 #if !defined(USE_FFMPEG)
 /// FFmpeg support may or may not be compiled in,
 /// but Preferences dialog requires this function nevertheless
@@ -83,11 +88,13 @@ bool LoadFFmpeg(bool showerror)
    {
       DropFFmpegLibs();
       gPrefs->Write(wxT("/FFmpeg/Enabled"), false);
+      gPrefs->Flush();
       return false;
    }
    else
    {
       gPrefs->Write(wxT("/FFmpeg/Enabled"), true);
+      gPrefs->Flush();
       return true;
    }
 }
@@ -98,12 +105,15 @@ void FFmpegStartup()
    bool enabled = false;
    gPrefs->Read(wxT("/FFmpeg/Enabled"),&enabled);
    // 'false' means that no errors should be shown whatsoever
-   if (enabled && !LoadFFmpeg(false))
+   if (!LoadFFmpeg(false))
    {
-     wxMessageBox(_("FFmpeg was configured in Preferences and successfully loaded before, \
-                      \nbut this time Audacity failed to load it at startup. \
-                      \n\nYou may want to go back to Preferences > Libraries and re-configure it."),
-                  _("FFmpeg startup failed"));
+      if (enabled)
+      {
+         wxMessageBox(_("FFmpeg was configured in Preferences and successfully loaded before, \
+                        \nbut this time Audacity failed to load it at startup. \
+                        \n\nYou may want to go back to Preferences > Libraries and re-configure it."),
+                      _("FFmpeg startup failed"));
+      }
    }
 }
 
@@ -153,7 +163,15 @@ void av_log_wx_callback(void* ptr, int level, const char* fmt, va_list vl)
    case 2: cpt = wxT("Debug"); break;
    default: cpt = wxT("Log"); break;
    }
-   wxLogMessage(wxT("%s: %s"),cpt.c_str(),printstring.c_str());
+#ifdef EXPERIMENTAL_OD_FFMPEG
+//if the decoding happens thru OD then this gets called from a non main thread, which means wxLogDebug
+//will crash.  
+//TODO:find some workaround for the log.  perhaps use ODManager as a bridge. for now just print
+   if(!wxThread::IsMain())
+      printf("%s: %s\n",(char*)cpt.char_str(),(char*)printstring.char_str());
+   else
+#endif
+      wxLogDebug(wxT("%s: %s"),cpt.c_str(),printstring.c_str());
 }
 
 //======================= Unicode aware uri protocol for FFmpeg
@@ -161,6 +179,7 @@ void av_log_wx_callback(void* ptr, int level, const char* fmt, va_list vl)
 
 static int ufile_open(URLContext *h, const char *filename, int flags)
 {
+   wxString name(strchr(filename, ':') + 1, wxConvUTF8);
    wxFile *f;
    wxFile::OpenMode mode;
 
@@ -169,15 +188,21 @@ static int ufile_open(URLContext *h, const char *filename, int flags)
       return AVERROR(ENOMEM);
    }
 
-   if (flags & URL_RDWR) {
+   // LLL:  These really should be logical AND tests, but on 2011/04/28, the URL_ open flags
+   //       changed in the FFmpeg source to values that were not compatible with previous
+   //       values.
+   //
+   //       Since Audacity doesn't use any other open flags (there aren't any others defined
+   //       anyway), making equality tests works for older and new FFmpeg headers.
+   if (flags == URL_RDWR) {
       mode = wxFile::read_write;
-   } else if (flags & URL_WRONLY) {
+   } else if (flags == URL_WRONLY) {
       mode = wxFile::write;
    } else {
       mode = wxFile::read;
    }
 
-   if (!f->Open((const wxChar *) filename, mode)) {
+   if (!f->Open(name, mode)) {
       delete f;
       return AVERROR(ENOENT);
    }
@@ -189,27 +214,27 @@ static int ufile_open(URLContext *h, const char *filename, int flags)
 
 static int ufile_read(URLContext *h, unsigned char *buf, int size)
 {
-   //return (int) ((wxFile *) h->priv_data)->Read(buf, size);
-   static int totalret = 0;
-   int ret = (int) ((wxFile *) h->priv_data)->Read(buf, size);
-   totalret += ret;
-   return ret;
+   return (int) ((wxFile *) h->priv_data)->Read(buf, size);
 }
 
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(52, 68, 0)
 static int ufile_write(URLContext *h, unsigned char *buf, int size)
+#else
+static int ufile_write(URLContext *h, const unsigned char *buf, int size)
+#endif
 {
    return (int) ((wxFile *) h->priv_data)->Write(buf, size);
 }
 
-#if LIBAVFORMAT_VERSION_MAJOR >= 52
 static int64_t ufile_seek(URLContext *h, int64_t pos, int whence)
-#else
-static offset_t ufile_seek(URLContext *h, offset_t pos, int whence)
-#endif
 {
-   wxSeekMode mode;
+   wxSeekMode mode = wxFromStart;
 
-   switch (whence)
+#if !defined(AVSEEK_FORCE)
+#define AVSEEK_FORCE 0
+#endif
+
+   switch (whence & ~AVSEEK_FORCE)
    {
    case (SEEK_SET):
      mode = wxFromStart;
@@ -240,7 +265,7 @@ static int ufile_close(URLContext *h)
 }
 
 URLProtocol ufile_protocol = {
-    "ufile",
+    UFILE_PROTOCOL,
     ufile_open,
     ufile_read,
     ufile_write,
@@ -249,23 +274,25 @@ URLProtocol ufile_protocol = {
 };
 
 // Open a file with a (possibly) Unicode filename
-int ufile_fopen(ByteIOContext **s, const wxString & name, int flags)
+int ufile_fopen(AVIOContext **s, const wxString & name, int flags)
 {
+   wxString url(wxString(wxT(UFILE_PROTOCOL)) + wxT(":") + name);
    URLContext *h;
    int err;
 
    // Open the file using our custom protocol and passing the (possibly) Unicode
-   // filename.  This is playing a slight trick since our ufile_open() routine above
-   // knows that the "char *" name may actually be wide characters.
-   err = FFmpegLibsInst->url_open_protocol(&h, &ufile_protocol, (const char *) name.c_str(), flags);
+   // filename.  We convert the name to UTF8 here and it will be converted back
+   // to original encoding in ufile_open().  This allows us to support Unicode
+   // filenames even though FFmpeg does not.
+   err = url_open(&h, (const char *) url.ToUTF8(), flags);
    if (err < 0) {
       return err;
    }
 
    // Associate the file with a context
-   err = FFmpegLibsInst->url_fdopen(s, h);
+   err = url_fdopen(s, h);
    if (err < 0) {
-      FFmpegLibsInst->url_close(h);
+      url_close(h);
       return err;
    }
 
@@ -285,23 +312,23 @@ int ufile_fopen_input(AVFormatContext **ic_ptr, wxString & name)
    wxCharBuffer fname;
    const char *filename;
    AVProbeData pd;
-   ByteIOContext *pb = NULL;
+   AVIOContext *pb = NULL;
    AVInputFormat *fmt = NULL;
    AVInputFormat *fmt1;
    int probe_size;
    int err;
 
    // Create a dummy file name using the extension from the original
-   f.SetName(wxT("ufile"));
+   f.SetName(wxT(UFILE_PROTOCOL));
    fname = f.GetFullName().mb_str();
    filename = (const char *) fname;
 
    // Initialize probe data...go ahead and preallocate the maximum buffer size.
    pd.filename = filename;
    pd.buf_size = 0;
-   pd.buf = (unsigned char *) FFmpegLibsInst->av_malloc(PROBE_BUF_MAX + AVPROBE_PADDING_SIZE);
+   pd.buf = (unsigned char *) av_malloc(PROBE_BUF_MAX + AVPROBE_PADDING_SIZE);
    if (pd.buf == NULL) {
-      err = AVERROR_NOMEM;
+      err = AVERROR(ENOMEM);
       goto fail;
    }
 
@@ -314,20 +341,27 @@ int ufile_fopen_input(AVFormatContext **ic_ptr, wxString & name)
       int score_max = probe_size < PROBE_BUF_MAX ? AVPROBE_SCORE_MAX / 4 : 0;
 
       // Read up to a "probe_size" worth of data
-      pd.buf_size = FFmpegLibsInst->get_buffer(pb, pd.buf, probe_size);
+      pd.buf_size = avio_read(pb, pd.buf, probe_size);
+
+      // AWD: with zero-length input files buf_size can come back negative;
+      // this causes problems so we might as well just fail
+      if (pd.buf_size < 0) {
+         err = AVERROR_INVALIDDATA;
+         goto fail;
+      }
 
       // Clear up to a "AVPROBE_PADDING_SIZE" worth of unused buffer
       memset(pd.buf + pd.buf_size, 0, AVPROBE_PADDING_SIZE);
 
       // Reposition file for succeeding scan
-      if (FFmpegLibsInst->url_fseek(pb, 0, SEEK_SET) < 0) {
+      if (avio_seek(pb, 0, SEEK_SET) < 0) {
          err = AVERROR(EIO);
          goto fail;
       }
 
       // Scan all input formats
       fmt = NULL;
-      for (fmt1 = FFmpegLibsInst->av_iformat_next(NULL); fmt1 != NULL; fmt1 = FFmpegLibsInst->av_iformat_next(fmt1)) {
+      for (fmt1 = av_iformat_next(NULL); fmt1 != NULL; fmt1 = av_iformat_next(fmt1)) {
          int score = 0;
 
          // Ignore the ones that are not file based
@@ -341,7 +375,7 @@ int ufile_fopen_input(AVFormatContext **ic_ptr, wxString & name)
          }
          // Otherwize, resort to extension matching if available
          else if (fmt1->extensions) {
-            if (FFmpegLibsInst->match_ext(filename, fmt1->extensions)) {
+            if (av_match_ext(filename, fmt1->extensions)) {
                score = 50;
             }
          }
@@ -359,28 +393,28 @@ int ufile_fopen_input(AVFormatContext **ic_ptr, wxString & name)
 
    // Didn't find a suitable format, so bail
    if (!fmt) {
-      err = AVERROR_NOFMT;
+      err = AVERROR(EILSEQ);
       goto fail;
    }
 
    // And finally, attempt to associate an input stream with the file
-   err = FFmpegLibsInst->av_open_input_stream(ic_ptr, pb, filename, fmt, NULL);
+   err = av_open_input_stream(ic_ptr, pb, filename, fmt, NULL);
    if (err) {
       goto fail;
    }
 
    // Done with the probe buffer
-   FFmpegLibsInst->av_freep(&pd.buf);
+   av_freep(&pd.buf);
 
    return 0;
 
 fail:
    if (pd.buf) {
-      FFmpegLibsInst->av_freep(&pd.buf);
+      av_freep(&pd.buf);
    }
 
    if (pb) {
-      FFmpegLibsInst->url_fclose(pb);
+      avio_close(pb);
    }
 
    *ic_ptr = NULL;
@@ -388,6 +422,136 @@ fail:
    return err;
 }
 
+streamContext *import_ffmpeg_read_next_frame(AVFormatContext* formatContext,
+                                             streamContext** streams,
+                                             unsigned int numStreams)
+{
+   streamContext *sc = NULL;
+   AVPacket pkt;
+
+   if (av_read_frame(formatContext,&pkt) < 0)
+   {
+      return NULL;
+   }
+
+   // Find a stream to which this frame belongs to
+   for (unsigned int i = 0; i < numStreams; i++)
+   {
+      if (streams[i]->m_stream->index == pkt.stream_index)
+         sc = streams[i];
+   }
+
+   // Off-stream packet. Don't panic, just skip it.
+   // When not all streams are selected for import this will happen very often.
+   if (sc == NULL)
+   {
+      av_free_packet(&pkt);
+      return (streamContext*)1;
+   }
+
+   // Copy the frame to the stream context
+   memcpy(&sc->m_pkt, &pkt, sizeof(AVPacket));
+
+   sc->m_pktValid = 1;
+   sc->m_pktDataPtr = pkt.data;
+   sc->m_pktRemainingSiz = pkt.size;
+
+   return sc;
+}
+
+int import_ffmpeg_decode_frame(streamContext *sc, bool flushing)
+{
+   int      nBytesDecoded;          
+   wxUint8 *pDecode = sc->m_pktDataPtr;
+   int      nDecodeSiz = sc->m_pktRemainingSiz;
+
+   sc->m_frameValid = 0;
+
+   if (flushing)
+   {
+      // If we're flushing the decoders we don't actually have any new data to decode.
+      pDecode = NULL;
+      nDecodeSiz = 0;
+   }
+   else
+   {
+      if (!sc->m_pktValid || (sc->m_pktRemainingSiz <= 0))
+      {
+         //No more data
+         return -1;
+      }
+   }
+
+   sc->m_samplefmt = sc->m_codecCtx->sample_fmt;
+   sc->m_samplesize = av_get_bits_per_sample_fmt(sc->m_samplefmt) / 8;
+
+   unsigned int newsize = FFMAX(sc->m_pkt.size * sc->m_samplesize, AVCODEC_MAX_AUDIO_FRAME_SIZE);
+   // Reallocate the audio sample buffer if it's smaller than the frame size.
+   if (newsize > sc->m_decodedAudioSamplesSiz )
+   {
+      if (sc->m_decodedAudioSamples)
+      {
+         av_free(sc->m_decodedAudioSamples);
+      }
+
+      sc->m_decodedAudioSamples = (uint8_t *) av_malloc(newsize);
+      sc->m_decodedAudioSamplesSiz = newsize;
+
+      if (sc->m_decodedAudioSamples == NULL)
+      {
+         //Can't allocate bytes
+         return -1;
+      }
+   }
+
+
+   sc->m_decodedAudioSamplesValidSiz = sc->m_decodedAudioSamplesSiz;
+
+#if LIBAVCODEC_VERSION_INT > AV_VERSION_INT(52, 25, 0)
+   // avcodec_decode_audio3() expects the size of the output buffer as the 3rd parameter but
+   // also returns the number of bytes it decoded in the same parameter.
+   AVPacket avpkt;
+   av_init_packet(&avpkt);
+   avpkt.data = pDecode;
+   avpkt.size = nDecodeSiz;
+   nBytesDecoded =
+      avcodec_decode_audio3(sc->m_codecCtx,
+                            (int16_t *)sc->m_decodedAudioSamples,    // out
+                            &sc->m_decodedAudioSamplesValidSiz,      // in/out
+                            &avpkt);                                 // in
+#else
+   // avcodec_decode_audio2() expects the size of the output buffer as the 3rd parameter but
+   // also returns the number of bytes it decoded in the same parameter.
+   nBytesDecoded =
+      avcodec_decode_audio2(sc->m_codecCtx, 
+                            (int16_t *) sc->m_decodedAudioSamples,   // out
+                            &sc->m_decodedAudioSamplesValidSiz,      // in/out
+                            pDecode,                                 // in
+                            nDecodeSiz);                             // in
+#endif
+   if (nBytesDecoded < 0)
+   {
+      // Decoding failed. Don't stop.
+      return -1;
+   }
+
+   // We may not have read all of the data from this packet. If so, the user can call again.
+   // Whether or not they do depends on if m_pktRemainingSiz == 0 (they can check).
+   sc->m_pktDataPtr += nBytesDecoded;
+   sc->m_pktRemainingSiz -= nBytesDecoded;
+
+   // At this point it's normally safe to assume that we've read some samples. However, the MPEG
+   // audio decoder is broken. If this is the case then we just return with m_frameValid == 0
+   // but m_pktRemainingSiz perhaps != 0, so the user can call again.
+   if (sc->m_decodedAudioSamplesValidSiz > 0)
+   {
+      sc->m_frameValid = 1;
+   }
+   return 0;
+}
+
+
+
 /*******************************************************/
 
 class FFmpegNotFoundDialog;
@@ -425,13 +589,13 @@ public:
       S.SetBorder(10);
       S.StartVerticalLay(true);
       {
-         text.Printf(_("Audacity needs the file %s to import and export audio via FFmpeg."), mName.c_str());
+         text.Printf(_("Audacity needs the file '%s' to import and export audio via FFmpeg."), mName.c_str());
          S.AddTitle(text);
 
          S.SetBorder(3);
          S.StartHorizontalLay(wxALIGN_LEFT, true);
          {
-            text.Printf(_("Location of %s:"), mName.c_str());
+            text.Printf(_("Location of '%s':"), mName.c_str());
             S.AddTitle(text);
          }
          S.EndHorizontalLay();
@@ -440,7 +604,7 @@ public:
          S.SetStretchyCol(0);
          {
             if (mLibPath.GetFullPath().IsEmpty()) {
-               text.Printf(_("To find %s, click here -->"), mName.c_str());
+               text.Printf(_("To find '%s', click here -->"), mName.c_str());
                mPathText = S.AddTextBox(wxT(""), text, 0);
             }
             else {
@@ -469,8 +633,8 @@ public:
       wxString question;
       /* i18n-hint: It's asking for the location of a file, for
       example, "Where is lame_enc.dll?" - you could translate
-      "Where would I find the file %s" instead if you want. */
-      question.Printf(_("Where is %s?"), mName.c_str());
+      "Where would I find the file '%s'?" instead if you want. */
+      question.Printf(_("Where is '%s'?"), mName.c_str());
 
       wxString path = FileSelector(question, 
          mLibPath.GetPath(),
@@ -551,7 +715,7 @@ bool FFmpegLibs::FindLibs(wxWindow *parent)
 
    wxLogMessage(wxT("Looking for FFmpeg libraries..."));
    if (!mLibAVFormatPath.IsEmpty()) {
-      wxLogMessage(wxT("mLibAVFormatPath is not empty, = %s"),mLibAVFormatPath.c_str());
+      wxLogMessage(wxT("mLibAVFormatPath ('%s') is not empty."), mLibAVFormatPath.c_str());
       wxFileName fn = mLibAVFormatPath;
       path = fn.GetPath();
       name = fn.GetFullName();
@@ -559,29 +723,31 @@ bool FFmpegLibs::FindLibs(wxWindow *parent)
    else {
       path = GetLibAVFormatPath();
       name = GetLibAVFormatName();
-      wxLogMessage(wxT("mLibAVFormatPath is empty, starting with path '%s', name '%s'"),path.c_str(),name.c_str());
+      wxLogMessage(wxT("mLibAVFormatPath is empty, starting with path '%s', name '%s'."), 
+                  path.c_str(), name.c_str());
    }
 
    FindFFmpegDialog fd(parent,
-      path,
-      name,
-      GetLibraryTypeString());
+                        path,
+                        name,
+                        GetLibraryTypeString());
 
    if (fd.ShowModal() == wxID_CANCEL) {
-      wxLogMessage(wxT("User canceled the dialog. Failed to find libraries."));
+      wxLogMessage(wxT("User canceled the dialog. Failed to find FFmpeg libraries."));
       return false;
    }
 
    path = fd.GetLibPath();
 
-   wxLogMessage(wxT("User-specified path = %s"),path.c_str());
+   wxLogMessage(wxT("User-specified path = '%s'"), path.c_str());
    if (!::wxFileExists(path)) {
-      wxLogMessage(wxT("User-specified file doesn't exist! Failed to find libraries."));
+      wxLogError(wxT("User-specified file does not exist. Failed to find FFmpeg libraries."));
       return false;
    }
-   wxLogMessage(wxT("User-specified file exists. Success."));
+   wxLogMessage(wxT("User-specified FFmpeg file exists. Success."));
    mLibAVFormatPath = path;
    gPrefs->Write(wxT("/FFmpeg/FFmpegLibPath"), mLibAVFormatPath);
+   gPrefs->Flush();
 
    return true;
 }
@@ -589,15 +755,15 @@ bool FFmpegLibs::FindLibs(wxWindow *parent)
 bool FFmpegLibs::LoadLibs(wxWindow *parent, bool showerr)
 {
 
-   wxLogMessage(wxT("Trying to load FFmpeg libraries"));
+   wxLogMessage(wxT("Trying to load FFmpeg libraries..."));
    if (ValidLibsLoaded()) {
-      wxLogMessage(wxT("Libraries already loaded - freeing"));
+      wxLogMessage(wxT("FFmpeg libraries are already loaded."));
       FreeLibs();
    }
 
    // First try loading it from a previously located path
    if (!mLibAVFormatPath.IsEmpty()) {
-      wxLogMessage(wxT("mLibAVFormatPath is not empty, = %s. Loading from it."),mLibAVFormatPath.c_str());
+      wxLogMessage(wxT("mLibAVFormatPath ('%s') is not empty. Loading from it."),mLibAVFormatPath.c_str());
       mLibsLoaded = InitLibs(mLibAVFormatPath,showerr);
    }
 
@@ -605,17 +771,30 @@ bool FFmpegLibs::LoadLibs(wxWindow *parent, bool showerr)
    if (!mLibsLoaded && !GetLibAVFormatPath().IsEmpty()) {
       wxFileName fn(GetLibAVFormatPath(), GetLibAVFormatName());
       wxString path = fn.GetFullPath();
-      wxLogMessage(wxT("Trying to load from default path %s."),path.c_str());
+      wxLogMessage(wxT("Trying to load FFmpeg libraries from default path, '%s'."), path.c_str());
       mLibsLoaded = InitLibs(path,showerr);
       if (mLibsLoaded) {
          mLibAVFormatPath = path;
       }
    }
+   
+#if defined(__WXMAC__)
+   // If not successful, try loading it from legacy path
+   if (!mLibsLoaded && !GetLibAVFormatPath().IsEmpty()) {
+      wxFileName fn(wxT("/usr/local/lib/audacity"), GetLibAVFormatName());
+      wxString path = fn.GetFullPath();
+      wxLogMessage(wxT("Trying to load FFmpeg libraries from legacy path, '%s'."), path.c_str());
+      mLibsLoaded = InitLibs(path,showerr);
+      if (mLibsLoaded) {
+         mLibAVFormatPath = path;
+      }
+   }
+#endif
 
    // If not successful, try loading using system search paths
    if (!ValidLibsLoaded()) {
       wxString path = GetLibAVFormatName();
-      wxLogMessage(wxT("Trying to load from system paths. File name is %s"),path.c_str());
+      wxLogMessage(wxT("Trying to load FFmpeg libraries from system paths. File name is '%s'."), path.c_str());
       mLibsLoaded = InitLibs(path,showerr);
       if (mLibsLoaded) {
          mLibAVFormatPath = path;
@@ -626,7 +805,7 @@ bool FFmpegLibs::LoadLibs(wxWindow *parent, bool showerr)
    /*
    if (!ValidLibsLoaded())
    {
-      wxLogMessage(wxT("Failed to load libraries altogether."));
+      wxLogError(wxT("Failed to load libraries altogether."));
       int dontShowDlg;
       FFmpegNotFoundDialog *dlg;
       gPrefs->Read(wxT("/FFmpeg/NotFoundDontShow"),&dontShowDlg,0);
@@ -640,11 +819,14 @@ bool FFmpegLibs::LoadLibs(wxWindow *parent, bool showerr)
    */
    // Oh well, just give up
    if (!ValidLibsLoaded()) {
-      if (showerr) wxMessageBox(_("Failed to find compatible FFmpeg libraries"));
+      wxString msg = _("Failed to find compatible FFmpeg libraries.");
+      if (showerr) 
+         wxMessageBox(msg);
+      wxLogError(msg);
       return false;
    }
 
-   wxLogMessage(wxT("Libraries loaded successfully!"));
+   wxLogMessage(wxT("FFmpeg libraries loaded successfully."));
    return true;
 }
 
@@ -655,238 +837,261 @@ bool FFmpegLibs::ValidLibsLoaded()
 
 bool FFmpegLibs::InitLibs(wxString libpath_format, bool showerr)
 {
-   // Initially we don't know where are the avcodec and avutl libs
-   wxDynamicLibrary *codec = NULL;
-   wxDynamicLibrary *util = NULL;
-   wxFileName name(libpath_format);
-
-   wxLogWindow* mLogger = wxGetApp().mLogger;
-
-   bool gotError = false;
-
-   wxString syspath;
-   bool pathfix = false;
-
    FreeLibs();
 
 #if defined(__WXMSW__)
-   wxLogMessage(wxT("Looking up PATH..."));
-   // First take PATH environment variable (store it's content)
+   wxString syspath;
+   bool pathfix = false;
+
+   wxLogMessage(wxT("Looking up PATH environment variable..."));
+   // First take PATH environment variable and store its content.
    if (wxGetEnv(wxT("PATH"),&syspath))
    {
-      wxLogMessage(wxT("PATH = %s"),syspath.c_str());
+      wxLogMessage(wxT("PATH = '%s'"), syspath.c_str());
       wxString fmtdirsc = wxPathOnly(libpath_format) + wxT(";");
       wxString scfmtdir = wxT(";") + wxPathOnly(libpath_format);
       wxString fmtdir = wxPathOnly(libpath_format);
-      wxLogMessage(wxT("Checking that %s is in PATH..."),fmtdir.c_str());
+      wxLogMessage(wxT("Checking that '%s' is in PATH..."), fmtdir.c_str());
       // If the directory, where libavformat is, is not in PATH - add it
       if (!syspath.Contains(fmtdirsc) && !syspath.Contains(scfmtdir) && !syspath.Contains(fmtdir))
       {
-         wxLogMessage(wxT("not in PATH!"));
+         wxLogWarning(wxT("FFmpeg directory is not in PATH."), fmtdir.c_str());
          if (syspath.Last() == wxT(';'))
          {
-            wxLogMessage(wxT("Appending %s ..."),fmtdir.c_str());
+            wxLogMessage(wxT("Temporarily appending '%s' to PATH..."), fmtdir.c_str());
             syspath.Append(fmtdirsc);
          }
          else
          {
-            wxLogMessage(wxT("Appending %s ..."),scfmtdir.c_str());
+            wxLogMessage(wxT("Temporarily appending '%s' to PATH..."), scfmtdir.c_str());
             syspath.Append(scfmtdir);
          }
 
          if (wxSetEnv(wxT("PATH"),syspath.c_str()))
-         {
             // Remember to change PATH back to normal after we're done
             pathfix = true;
-         }
          else
-         {
-            wxLogMessage(wxT("wxSetEnv(%s) failed."),syspath.c_str());
-         }
+            wxLogSysError(wxT("Setting PATH via wxSetEnv('%s') failed."),syspath.c_str());
       }
       else
       {
-         wxLogMessage(wxT("in PATH."));
+         wxLogMessage(wxT("FFmpeg directory is in PATH."));
       }
    }
    else
    {
-      wxLogMessage(wxT("PATH does not exist."));
+      wxLogSysError(wxT("PATH does not exist."));
    }
 #endif
 
    //Load libavformat
-   avformat = new wxDynamicLibrary();
-   wxLogMessage(wxT("Loading avformat from %s"),libpath_format.c_str());
-   if (showerr)
-      mLogger->SetActiveTarget(NULL);
-   gotError = !avformat->Load(libpath_format, wxDL_LAZY);
-   if (showerr)
-      mLogger->SetActiveTarget(mLogger);
+   // Initially we don't know where are the avcodec and avutl libs
+   wxDynamicLibrary *codec = NULL;
+   wxDynamicLibrary *util = NULL;
+   wxFileName name(libpath_format);
+   bool gotError = false;
 
+   // Check for a monolithic avformat
+   avformat = new wxDynamicLibrary();
+   wxLogMessage(wxT("Checking for monolithic avformat from '%s'."), name.GetFullPath().c_str());
+   gotError = !avformat->Load(name.GetFullPath(), wxDL_LAZY);
+
+   // Verify it really is monolithic
    if (!gotError) {
-      if (avformat->HasSymbol(wxT("av_free"))) {
-         util = avformat;
+      wxFileName actual;
+
+      actual = FileNames::PathFromAddr(avformat->GetSymbol(wxT("avutil_version")));
+      if (actual.GetFullPath().IsSameAs(name.GetFullPath())) {
+         actual = FileNames::PathFromAddr(avformat->GetSymbol(wxT("avcodec_version")));
+         if (actual.GetFullPath().IsSameAs(name.GetFullPath())) {
+             util = avformat;
+             codec = avformat;
+         }
       }
-      if (avformat->HasSymbol(wxT("avcodec_init"))) {
-         codec = avformat;
+
+      if (util == NULL || codec == NULL) {
+         wxLogMessage(wxT("avformat not monolithic"));
+         avformat->Unload();
+         util = NULL;
+         codec = NULL;
+      }
+      else {
+         wxLogMessage(wxT("avformat is monolithic"));
       }
    }
 
    if (!util) {
       name.SetFullName(GetLibAVUtilName());
-      avutil = util =  new wxDynamicLibrary();
-      wxLogMessage(wxT("Loading avutil from %s"),name.GetFullPath().c_str());
-      if (showerr)
-         mLogger->SetActiveTarget(NULL);
+      avutil = util = new wxDynamicLibrary();
+      wxLogMessage(wxT("Loading avutil from '%s'."), name.GetFullPath().c_str());
       util->Load(name.GetFullPath(), wxDL_LAZY);
-      if (showerr)
-         mLogger->SetActiveTarget(mLogger);
    }
 
    if (!codec) {
       name.SetFullName(GetLibAVCodecName());
       avcodec = codec = new wxDynamicLibrary();
-      wxLogMessage(wxT("Loading avcodec from %s"),name.GetFullPath().c_str());
-      if (showerr)
-         mLogger->SetActiveTarget(NULL);
+      wxLogMessage(wxT("Loading avcodec from '%s'."), name.GetFullPath().c_str());
       codec->Load(name.GetFullPath(), wxDL_LAZY);
-      if (showerr)
-         mLogger->SetActiveTarget(mLogger);
    }
 
    if (!avformat->IsLoaded()) {
-      if (showerr)
-         mLogger->SetActiveTarget(NULL);
-      gotError = !avformat->Load(libpath_format, wxDL_LAZY);
-      if (showerr)
-         mLogger->SetActiveTarget(mLogger);
+      name.SetFullName(libpath_format);
+      wxLogMessage(wxT("Loading avformat from '%s'."), name.GetFullPath().c_str());
+      gotError = !avformat->Load(name.GetFullPath(), wxDL_LAZY);
    }
 
+#if defined(__WXMSW__)
    //Return PATH to normal
    if ( pathfix )
    {
       wxString oldpath = syspath.BeforeLast(wxT(';'));
-      wxLogMessage(wxT("Returning PATH to normal..."));
+      wxLogMessage(wxT("Returning PATH to previous setting..."));
       wxSetEnv(wxT("PATH"),oldpath.c_str());
    }
+#endif
 
    if (gotError) {
-      wxLogMessage(wxT("Failed to load FFmpeg libs"));
+      wxLogError(wxT("Failed to load FFmpeg libraries."));
       FreeLibs();
       return false;
    }
 
+   // Show the actual libraries loaded
+   if (avutil) {
+      wxLogMessage(wxT("Actual avutil path %s"),
+                 FileNames::PathFromAddr(avutil->GetSymbol(wxT("avutil_version"))).c_str());
+   }
+   if (avcodec) {
+      wxLogMessage(wxT("Actual avcodec path %s"),
+                 FileNames::PathFromAddr(avcodec->GetSymbol(wxT("avcodec_version"))).c_str());
+   }
+   if (avformat) {
+      wxLogMessage(wxT("Actual avformat path %s"),
+                 FileNames::PathFromAddr(avformat->GetSymbol(wxT("avformat_version"))).c_str());
+   }
+
    wxLogMessage(wxT("Importing symbols..."));
-   INITDYN(avformat,av_register_all);
-   INITDYN(avformat,av_open_input_file);
-   INITDYN(avformat,av_find_stream_info);
-   INITDYN(avformat,av_read_frame);
-   INITDYN(avformat,av_seek_frame);
-   INITDYN(avformat,av_close_input_file);
-   INITDYN(avformat,av_index_search_timestamp);
-   INITDYN(avformat,av_write_header);
-   INITDYN(avformat,av_interleaved_write_frame);
-   INITDYN(avformat,av_write_frame);
-   INITDYN(avformat,av_iformat_next);
-   INITDYN(avformat,av_oformat_next);
-   INITDYN(avformat,av_set_parameters);
-#if LIBAVFORMAT_VERSION_MAJOR < 53
-   INITDYN(avformat,register_protocol);
-   av_register_protocol = register_protocol;
+   FFMPEG_INITDYN(avformat, av_register_all);
+   FFMPEG_INITDYN(avformat, av_find_stream_info);
+   FFMPEG_INITDYN(avformat, av_read_frame);
+   FFMPEG_INITDYN(avformat, av_seek_frame);
+   FFMPEG_INITDYN(avformat, av_close_input_file);
+   FFMPEG_INITDYN(avformat, av_write_header);
+   FFMPEG_INITDYN(avformat, av_interleaved_write_frame);
+   FFMPEG_INITDYN(avformat, av_iformat_next);
+   FFMPEG_INITDYN(avformat, av_oformat_next);
+   FFMPEG_INITDYN(avformat, av_set_parameters);
+   FFMPEG_INITDYN(avformat, url_open_protocol);
+   FFMPEG_INITDYN(avformat, url_open);
+   FFMPEG_INITDYN(avformat, url_fdopen);
+   FFMPEG_INITDYN(avformat, url_close);
+   FFMPEG_INITDYN(avformat, url_fseek);
+   FFMPEG_INITDYN(avformat, url_fclose);
+   FFMPEG_INITDYN(avformat, av_new_stream);
+   FFMPEG_INITDYN(avformat, avformat_alloc_context);
+   FFMPEG_INITDYN(avformat, av_write_trailer);
+   FFMPEG_INITDYN(avformat, av_codec_get_tag);
+   FFMPEG_INITDYN(avformat, avformat_version);
+   FFMPEG_INITDYN(avformat, av_open_input_stream);
+   FFMPEG_INITDYN(avformat, av_metadata_get);
+
+   FFMPEG_INITALT(avformat, av_register_protocol2, av_register_protocol);
+   FFMPEG_INITALT(avformat, avio_read, get_buffer);
+   FFMPEG_INITALT(avformat, avio_seek, url_fseek);
+   FFMPEG_INITALT(avformat, avio_close, url_fclose);
+   FFMPEG_INITALT(avformat, av_metadata_set2, av_metadata_set);
+   FFMPEG_INITALT(avformat, av_guess_format, guess_format);
+   FFMPEG_INITALT(avformat, av_match_ext, match_ext);
+
+#if LIBAVCODEC_VERSION_INT > AV_VERSION_INT(52, 58, 0)
+   FFMPEG_INITDYN(avcodec, av_init_packet);
 #else
-   INITDYN(avformat,av_register_protocol);
+   FFMPEG_INITDYN(avformat, av_init_packet);
 #endif
-   INITDYN(avformat,url_open_protocol);
-   INITDYN(avformat,url_fdopen);
-   INITDYN(avformat,url_close);
-   INITDYN(avformat,url_fopen);
-   INITDYN(avformat,url_fseek);
-   INITDYN(avformat,url_fclose);
-   INITDYN(avformat,url_fsize);
-   INITDYN(avformat,av_new_stream);
-   INITDYN(avformat,av_alloc_format_context);
-   INITDYN(avformat,guess_format);
-   INITDYN(avformat,av_write_trailer);
-   INITDYN(avformat,av_codec_get_id);
-   INITDYN(avformat,av_codec_get_tag);
-   INITDYN(avformat,avformat_version);
-   INITDYN(avformat,av_open_input_file);
-   INITDYN(avformat,av_open_input_stream);
-   INITDYN(avformat,get_buffer);
-   INITDYN(avformat,match_ext);
 
-   INITDYN(codec,avcodec_init);
-   INITDYN(codec,av_init_packet);
-   INITDYN(codec,av_free_packet);
-   INITDYN(codec,avcodec_find_encoder);
-   INITDYN(codec,avcodec_find_encoder_by_name);
-   INITDYN(codec,avcodec_find_decoder);
-   INITDYN(codec,avcodec_find_decoder_by_name);
-   INITDYN(codec,avcodec_string);
-   INITDYN(codec,avcodec_get_context_defaults);
-   INITDYN(codec,avcodec_alloc_context);
-   INITDYN(codec,avcodec_get_frame_defaults);
-   INITDYN(codec,avcodec_alloc_frame);
-   INITDYN(codec,avcodec_open);
-   INITDYN(codec,avcodec_decode_audio2);
-   INITDYN(codec,avcodec_encode_audio);
-   INITDYN(codec,avcodec_close);
-   INITDYN(codec,avcodec_register_all);
-   INITDYN(codec,avcodec_flush_buffers);
-   INITDYN(codec,av_get_bits_per_sample);
-   INITDYN(codec,av_get_bits_per_sample_format);
-   INITDYN(codec,avcodec_version);
-   INITDYN(codec,av_fast_realloc);
-   INITDYN(codec,av_codec_next);
+#if LIBAVFORMAT_VERSION_INT > AV_VERSION_INT(52, 31, 0)
+   FFMPEG_INITDYN(avcodec, av_free_packet);
+#endif
+   FFMPEG_INITDYN(avcodec, avcodec_init);
+   FFMPEG_INITDYN(avcodec, avcodec_find_encoder);
+   FFMPEG_INITDYN(avcodec, avcodec_find_encoder_by_name);
+   FFMPEG_INITDYN(avcodec, avcodec_find_decoder);
+   FFMPEG_INITDYN(avcodec, avcodec_get_context_defaults);
+   FFMPEG_INITDYN(avcodec, avcodec_open);
+#if LIBAVCODEC_VERSION_INT > AV_VERSION_INT(52, 25, 0)
+   FFMPEG_INITDYN(avcodec, avcodec_decode_audio3);
+#else
+   FFMPEG_INITDYN(avcodec, avcodec_decode_audio2);
+#endif
+   FFMPEG_INITDYN(avcodec, avcodec_encode_audio);
+   FFMPEG_INITDYN(avcodec, avcodec_close);
+   FFMPEG_INITDYN(avcodec, avcodec_register_all);
+   FFMPEG_INITDYN(avcodec, avcodec_version);
+   FFMPEG_INITDYN(avcodec, av_fast_realloc);
+   FFMPEG_INITDYN(avcodec, av_codec_next);
+   FFMPEG_INITDYN(avcodec, av_get_bits_per_sample_format);
 
-   INITDYN(util,av_free);
-   INITDYN(util,av_log_set_callback);
-   INITDYN(util,av_log_default_callback);
-   INITDYN(util,av_fifo_alloc);
-   INITDYN(util,av_fifo_free);
-   INITDYN(util,av_fifo_generic_read);
-   INITDYN(util,av_fifo_size);
-   INITDYN(util,av_fifo_generic_write);
-   INITDYN(util,av_fifo_realloc2);
-   INITDYN(util,av_malloc);
-   INITDYN(util,av_freep);
-   INITDYN(util,av_rescale_q);
-   INITDYN(util,av_strstart);
-   INITDYN(util,avutil_version);
+   FFMPEG_INITALT(avcodec, av_get_bits_per_sample_fmt, av_get_bits_per_sample_format);
+
+   FFMPEG_INITDYN(avutil, av_free);
+   FFMPEG_INITDYN(avutil, av_log_set_callback);
+   FFMPEG_INITDYN(avutil, av_log_default_callback);
+#if LIBAVUTIL_VERSION_INT > AV_VERSION_INT(49, 15, 0)
+   FFMPEG_INITDYN(avutil, av_fifo_alloc);
+#else
+   FFMPEG_INITDYN(avutil, av_fifo_init);
+#endif
+   FFMPEG_INITDYN(avutil, av_fifo_generic_read);
+   FFMPEG_INITDYN(avutil, av_fifo_realloc2);
+   FFMPEG_INITDYN(avutil, av_fifo_free);
+   FFMPEG_INITDYN(avutil, av_fifo_size);
+   FFMPEG_INITDYN(avutil, av_malloc);
+   FFMPEG_INITDYN(avutil, av_fifo_generic_write);
+   FFMPEG_INITDYN(avutil, av_freep);
+   FFMPEG_INITDYN(avutil, av_rescale_q);
+   FFMPEG_INITDYN(avutil, avutil_version);
 
    //FFmpeg initialization
    wxLogMessage(wxT("All symbols loaded successfully. Initializing the library."));
-   this->avcodec_init();
-   this->avcodec_register_all();
-   this->av_register_all();
+   avcodec_init();
+   avcodec_register_all();
+   av_register_all();
    
-   wxLogMessage(wxT("Retrieving library version."));
-   int avcver = this->avcodec_version();
-   int avfver = this->avformat_version();
-   int avuver = this->avutil_version();
+   wxLogMessage(wxT("Retrieving FFmpeg library version numbers:"));
+   int avfver = avformat_version();
+   int avcver = avcodec_version();
+   int avuver = avutil_version();
    mAVCodecVersion = wxString::Format(wxT("%d.%d.%d"),avcver >> 16 & 0xFF, avcver >> 8 & 0xFF, avcver & 0xFF);
    mAVFormatVersion = wxString::Format(wxT("%d.%d.%d"),avfver >> 16 & 0xFF, avfver >> 8 & 0xFF, avfver & 0xFF);
    mAVUtilVersion = wxString::Format(wxT("%d.%d.%d"),avuver >> 16 & 0xFF, avuver >> 8 & 0xFF, avuver & 0xFF);
 
-   wxLogMessage(wxT("AVCodec version 0x%06x - %s (built against 0x%06x - %s)"),avcver,mAVCodecVersion.c_str(),LIBAVCODEC_VERSION_INT,wxString::FromUTF8(AV_STRINGIFY(LIBAVCODEC_VERSION)).c_str());
-   wxLogMessage(wxT("AVFormat version 0x%06x - %s (built against 0x%06x - %s)"),avfver,mAVFormatVersion.c_str(),LIBAVFORMAT_VERSION_INT,wxString::FromUTF8(AV_STRINGIFY(LIBAVFORMAT_VERSION)).c_str());
-   wxLogMessage(wxT("AVUtil version 0x%06x - %s (built against 0x%06x - %s)"),avuver,mAVUtilVersion.c_str(),LIBAVUTIL_VERSION_INT,wxString::FromUTF8(AV_STRINGIFY(LIBAVUTIL_VERSION)).c_str());
+   wxLogMessage(wxT("   AVCodec version 0x%06x - %s (built against 0x%06x - %s)"), 
+                  avcver, mAVCodecVersion.c_str(), LIBAVCODEC_VERSION_INT, 
+                  wxString::FromUTF8(AV_STRINGIFY(LIBAVCODEC_VERSION)).c_str());
+   wxLogMessage(wxT("   AVFormat version 0x%06x - %s (built against 0x%06x - %s)"), 
+                  avfver, mAVFormatVersion.c_str(), LIBAVFORMAT_VERSION_INT, 
+                  wxString::FromUTF8(AV_STRINGIFY(LIBAVFORMAT_VERSION)).c_str());
+   wxLogMessage(wxT("   AVUtil version 0x%06x - %s (built against 0x%06x - %s)"), 
+                  avuver,mAVUtilVersion.c_str(), LIBAVUTIL_VERSION_INT, 
+                  wxString::FromUTF8(AV_STRINGIFY(LIBAVUTIL_VERSION)).c_str());
 
    int avcverdiff = (avcver >> 16 & 0xFF) - int(LIBAVCODEC_VERSION_MAJOR);
    int avfverdiff = (avfver >> 16 & 0xFF) - int(LIBAVFORMAT_VERSION_MAJOR);
    int avuverdiff = (avuver >> 16 & 0xFF) - int(LIBAVUTIL_VERSION_MAJOR);
-   wxLogMessage(wxT("AVCodec version mismatch is %d"),avcverdiff);
-   wxLogMessage(wxT("AVFormat version mismatch is %d"),avfverdiff);
-   wxLogMessage(wxT("AVUtil version mismatch is %d"),avuverdiff);
+   if (avcverdiff != 0)
+      wxLogError(wxT("AVCodec version mismatch = %d"), avcverdiff);
+   if (avfverdiff != 0)
+      wxLogError(wxT("AVFormat version mismatch = %d"), avfverdiff);
+   if (avuverdiff != 0)
+      wxLogError(wxT("AVUtil version mismatch = %d"), avuverdiff);
    //make sure that header and library major versions are the same
    if (avcverdiff != 0 || avfverdiff != 0 || avuverdiff != 0)
    {
-      wxLogMessage(wxT("Version mismatch! Libraries are unusable."));
+      wxLogError(wxT("Version mismatch. FFmpeg libraries are unusable."));
       return false;
    }
 
-   av_register_protocol(&ufile_protocol);
+   av_register_protocol2(&ufile_protocol, sizeof(ufile_protocol));
 
    return true;
 }
