@@ -22,6 +22,7 @@ i.e. an alternative to the usual interface, for Audacity.
 #include <wx/list.h>
 #include <wx/log.h>
 #include <wx/string.h>
+#include <wx/filename.h>
 
 #include "Audacity.h"
 #include "AudacityApp.h"
@@ -30,6 +31,7 @@ i.e. an alternative to the usual interface, for Audacity.
 #include "commands/ScriptCommandRelay.h"
 #include <NonGuiThread.h>  // header from libwidgetextra
 
+#include "Prefs.h"
 #include "LoadModules.h"
 
 #define initFnName      "ExtensionModuleInit"
@@ -39,7 +41,8 @@ i.e. an alternative to the usual interface, for Audacity.
 
 typedef wxWindow * pwxWindow;
 typedef int (*tModuleInit)(int);
-typedef wxString (*tVersionFn)();
+//typedef wxString (*tVersionFn)();
+typedef wxChar * (*tVersionFn)();
 typedef pwxWindow (*tPanelFn)(int);
 
 // This variable will hold the address of a subroutine in 
@@ -70,10 +73,33 @@ wxWindow * MakeHijackPanel()
 // starts a thread and reads script commands.
 tpRegScriptServerFunc scriptFn;
 
+bool IsAllowedModule( wxString fname )
+{
+   bool bLoad = false;
+   wxString ShortName = wxFileName( fname ).GetName();
+   if( (ShortName.CmpNoCase( wxT("mod-script-pipe")) == 0 ))
+   {
+      gPrefs->Read(wxT("/Module/mod-script-pipe"), &bLoad, false);
+   }
+   else if( (ShortName.CmpNoCase( wxT("mod-nyq-bench")) == 0 ))
+   {
+      gPrefs->Read(wxT("/Module/mod-nyq-bench"), &bLoad, false);
+   }
+   else if( (ShortName.CmpNoCase( wxT("mod-track-panel")) == 0 ))
+   {
+      gPrefs->Read(wxT("/Module/mod-track-panel"), &bLoad, false);
+   }
+   return bLoad;
+}
+
 void LoadModule(wxString fname)
 {
-   wxLogDebug(wxT("About to load %s"), fname.c_str() );
-   wxLogNull logNo;
+   if( !IsAllowedModule( fname ) )
+      return;
+
+   wxLogDebug(wxT("About to load module %s"), fname.c_str());
+   wxLogNull logNo; // Don't show wxWidgets Error if cannot load within this method. (Fix bug 544.)
+
    tModuleInit mainFn = NULL;
 
    // As a courtesy to some modules that might be bridges to
@@ -87,36 +113,13 @@ void LoadModule(wxString fname)
    wxDynamicLibrary* pDLL = new wxDynamicLibrary();
    if (pDLL && pDLL->Load(fname, wxDL_LAZY)) 
    {
-      int result = 1;
-      mainFn   = (tModuleInit)(pDLL->GetSymbol(wxT(initFnName)));
-
-      if (mainFn) 
-         result = mainFn( 0 );
-
-      // If the module provides a version string, check that it matches the
-      // Audacity version string. (For now, they must match exactly)
-      // For compatibility reasons, if no version string is provided we try to
-      // load the module anyway.
-      tVersionFn versionFn = (tVersionFn)(pDLL->GetSymbol(wxT(versionFnName)));
-      if (versionFn)
-      {
-         wxString moduleVersion = versionFn();
-         if (moduleVersion != AUDACITY_VERSION_STRING)
-         {
-            wxLogDebug(wxT("The module %s is designed to work with Audacity version %s; it will not be loaded."), fname.c_str(), moduleVersion.c_str());
-            delete pDLL;
-            return;
-         }
-      }
-      else
-      {
-         wxLogDebug(wxT("The module %s does not provide a version string. Attempting to load it anyway."), fname.c_str());
-      }
-
-      if(( scriptFn == NULL ) &&(result>=0 ))
+      // We've loaded and initialised OK.
+      // So look for special case functions:
+      // (a) for scripting.
+      if( scriptFn == NULL )
          scriptFn = (tpRegScriptServerFunc)(pDLL->GetSymbol(wxT(scriptFnName)));
-
-      if((pPanelHijack==NULL ) && (result>=0))
+      // (b) for hijacking the entire Audacity panel.
+      if( pPanelHijack==NULL )
          pPanelHijack = (tPanelFn)(pDLL->GetSymbol(wxT(mainPanelFnName)));
    }
 
@@ -194,9 +197,22 @@ bool Module::Load()
       return false;
    }
 
-   mDispatch = (fnModuleDispatch) mLib->GetSymbol(wxT(ModuleDispatchName));
+   // Check version string matches.  (For now, they must match exactly)
+   tVersionFn versionFn = (tVersionFn)(mLib->GetSymbol(wxT(versionFnName)));
+   if (versionFn == NULL){
+      wxLogError(wxT("The module %s does not provide a version string. It will not be loaded."), mName.c_str());
+      return false;
+   }
 
+   wxString moduleVersion = versionFn();
+   if( !moduleVersion.IsSameAs(AUDACITY_VERSION_STRING)) {
+      wxLogError(wxT("The module %s is designed to work with Audacity version %s; it will not be loaded."), mName.c_str(), moduleVersion.c_str());
+      return false;
+   }
+
+   mDispatch = (fnModuleDispatch) mLib->GetSymbol(wxT(ModuleDispatchName));
    if (!mDispatch) {
+      // Module does not provide a dispacth function...
       return false;
    }
 
@@ -206,7 +222,6 @@ bool Module::Load()
    }
 
    mDispatch = NULL;
-
    return false;
 }
 
@@ -258,6 +273,7 @@ void ModuleManager::Initialize()
    wxString pathVar;
    size_t i;
 
+   // JKC: Is this code duplicating LoadModules() ????
    // Code from LoadLadspa that might be useful in load modules.
    pathVar = wxGetenv(wxT("AUDACITY_MODULES_PATH"));
    if (pathVar != wxT("")) {
@@ -279,13 +295,16 @@ void ModuleManager::Initialize()
    #endif
 
    for (i = 0; i < files.GetCount(); i++) {
-      Module *module = new Module(files[i]);
+      if( IsAllowedModule( files[i] ) )
+      {
+         Module *module = new Module(files[i]);
 
-      if (module->Load()) {
-         mInstance->mModules.Add(module);
-      }
-      else {
-         delete module;
+         if (module->Load()) {
+            mInstance->mModules.Add(module);
+         }
+         else {
+            delete module;
+         }
       }
    }
 }
