@@ -39,7 +39,7 @@ WX_DEFINE_USER_EXPORTED_ARRAY(WaveTrack*, WaveTrackArray, class AUDACITY_DLL_API
 
 #if defined(USE_MIDI)
 class NoteTrack;
-WX_DEFINE_ARRAY(NoteTrack*, NoteTrackArray);
+WX_DEFINE_USER_EXPORTED_ARRAY(NoteTrack*, NoteTrackArray, class AUDACITY_DLL_API);
 #endif
 
 class TrackList;
@@ -104,7 +104,7 @@ class AUDACITY_DLL_API Track: public XMLTagHandler
       MonoChannel = 2
    };
 
-   enum
+   enum TrackKindEnum
    {
       None,
       Wave,
@@ -114,7 +114,7 @@ class AUDACITY_DLL_API Track: public XMLTagHandler
       Label,
       Time,
       All
-   } TrackKindEnum;
+   };
 
    Track(DirManager * projDirManager);
    Track(const Track &orig);
@@ -124,7 +124,7 @@ class AUDACITY_DLL_API Track: public XMLTagHandler
    void Init(const Track &orig);
    virtual Track *Duplicate() = 0;
 
-   // Called when this track is merged with another, and should
+   // Called when this track is merged to stereo with another, and should
    // take on some paramaters of its partner.
    virtual void Merge(const Track &orig);
 
@@ -162,6 +162,10 @@ class AUDACITY_DLL_API Track: public XMLTagHandler
    virtual bool Clear(double t0, double t1) {return false;}
    virtual bool Paste(double t, Track * src) {return false;}
 
+   // This can be used to adjust a sync-lock selected track when the selection
+   // is replaced by one of a different length.
+   virtual bool SyncLockAdjust(double oldT1, double newT1);
+
    virtual bool Silence(double t0, double t1) {return false;}
    virtual bool InsertSilence(double t, double len) {return false;}
 
@@ -179,6 +183,9 @@ class AUDACITY_DLL_API Track: public XMLTagHandler
 
    virtual double GetStartTime() { return 0.0; }
    virtual double GetEndTime() { return 0.0; }
+
+   // Checks if sync-lock is on and any track in its sync-lock group is selected.
+   bool IsSyncLockSelected();
 };
 
 struct TrackListNode
@@ -198,7 +205,8 @@ class AUDACITY_DLL_API TrackListIterator
    virtual Track *First(TrackList * val = NULL);
    virtual Track *StartWith(Track * val);
    virtual Track *Next(bool skiplinked = false);
-   virtual Track *Last();
+   virtual Track *Prev(bool skiplinked = false);
+   virtual Track *Last(bool skiplinked = false);
 
    Track *ReplaceCurrent(Track *t);                // returns original
    Track *RemoveCurrent(bool deletetrack = false); // returns next
@@ -208,20 +216,39 @@ class AUDACITY_DLL_API TrackListIterator
    TrackListNode *cur;
 };
 
+// TrackListCondIterator (base class for iterators that iterate over all tracks)
+// that meet a condition)
+class AUDACITY_DLL_API TrackListCondIterator: public TrackListIterator
+{
+   public:
+      TrackListCondIterator(TrackList *val = NULL)
+         :  TrackListIterator(val) {};
+      virtual ~TrackListCondIterator() {};
+
+      // Iteration functions
+      Track *First(TrackList *val = NULL);
+      Track *StartWith(Track *val);
+      Track *Next(bool skiplinked = false);
+      Track *Prev(bool skiplinked = false);
+      Track *Last(bool skiplinked = false);
+
+   protected:
+      virtual bool Condition(Track *t) = 0;
+};
+
 //
 // TrackListOfKindIterator
 //
 // Based on TrackListIterator and returns only tracks of the specified type.
 //
-class AUDACITY_DLL_API TrackListOfKindIterator: public TrackListIterator
+class AUDACITY_DLL_API TrackListOfKindIterator: public TrackListCondIterator
 {
  public:
    TrackListOfKindIterator(int kind, TrackList * val = NULL);
    virtual ~TrackListOfKindIterator() {};
 
-   // Iterate functions
-   Track *First(TrackList * val = NULL);
-   Track *Next(bool skiplinked = false);
+ protected:
+   virtual bool Condition(Track *t);
 
  private:
    int kind;
@@ -236,14 +263,10 @@ class AUDACITY_DLL_API SelectedTrackListOfKindIterator: public TrackListOfKindIt
 {
  public:
     SelectedTrackListOfKindIterator(int kind, TrackList * val = NULL) : TrackListOfKindIterator(kind, val) {};
-	virtual ~SelectedTrackListOfKindIterator() {};
+   virtual ~SelectedTrackListOfKindIterator() {};
 
-   // Iterate functions
-   Track *First(TrackList * val = NULL);
-   Track *Next(bool skiplinked = false);
-
- private:
-   int kind;
+ protected:
+   bool Condition(Track *t);
 };
 
 //
@@ -251,54 +274,39 @@ class AUDACITY_DLL_API SelectedTrackListOfKindIterator: public TrackListOfKindIt
 //
 // Based on TrackListIterator returns only the currently visible tracks.
 //
-class AUDACITY_DLL_API VisibleTrackIterator: public TrackListIterator
+class AUDACITY_DLL_API VisibleTrackIterator: public TrackListCondIterator
 {
  public:
    VisibleTrackIterator(AudacityProject *project);
    virtual ~VisibleTrackIterator() {};
 
-   // Iterate functions
-   Track *First();
-   Track *Next(bool skiplinked = false);
+ protected:
+   bool Condition(Track *t);
 
  private:
    AudacityProject *mProject;
    wxRect mPanelRect;
 };
 
-//
-// TrackGroupIterator
-//
-// Based on TrackListIterator returns only tracks belonging to the group
+
+// SyncLockedTracksIterator returns only tracks belonging to the sync-locked tracks 
 // in which the starting track is a member.
-//
-class AUDACITY_DLL_API TrackGroupIterator: public TrackListIterator
+class AUDACITY_DLL_API SyncLockedTracksIterator : public TrackListIterator
 {
  public:
-   TrackGroupIterator(TrackList * val);
-   virtual ~TrackGroupIterator() {};
+   SyncLockedTracksIterator(TrackList * val);
+   virtual ~SyncLockedTracksIterator() {};
 
    // Iterate functions
    Track *First(Track *member);
    Track *Next(bool skiplinked = false);
+   Track *Prev(bool skiplinked = false);
+   Track *Last(bool skiplinked = false);
 
  private:
-   bool mEndOfGroup;
+   bool mInLabelSection;
 };
 
-//
-// TrackAndGroupIterator
-//
-// Based on TrackListIterator has methods to retrieve both tracks and groups
-//
-class AUDACITY_DLL_API TrackAndGroupIterator: public TrackListIterator
-{
- public:
-   TrackAndGroupIterator(TrackList * val);
-
-   // Iterate functions
-   Track *NextGroup(bool skiplinked = false);
-};
 
 /** \brief TrackList is a flat linked list of tracks supporting Add,  Remove,
  * Clear, and Contains, plus serialization of the list of tracks.
@@ -327,7 +335,8 @@ class AUDACITY_DLL_API TrackList:public wxEvtHandler
    friend class Track;
    friend class TrackListIterator;
 
-   /// Add this Track or all children of this TrackGroup
+   /// Add this Track or all children of this TrackGroup 
+   // FIX-ME: What does that comment mean by "TrackGroup"? There was no such class at tag Audacity_1_3_13.
    void Add(Track * t);
    void AddToHead(Track * t);
 
@@ -335,6 +344,7 @@ class AUDACITY_DLL_API TrackList:public wxEvtHandler
    void Replace(Track * t, Track * with, bool deletetrack = false);
 
    /// Remove this Track or all children of this TrackGroup
+   // FIX-ME: What does that comment mean by "TrackGroup"? There was no such class at tag Audacity_1_3_13.
    void Remove(Track * t, bool deletetrack = false);
 
    /// Make the list empty
@@ -348,6 +358,11 @@ class AUDACITY_DLL_API TrackList:public wxEvtHandler
    Track *GetLink(Track * t) const;
 
    Track *GetPrev(Track * t, bool linked = false) const;
+   
+   /** Return a track in the list that comes after Track t
+     * @param t a track in the list
+     * @param linked if true, skips over linked tracks, if false returns the next track even if it is a linked track
+    **/
    Track *GetNext(Track * t, bool linked = false) const;
    int GetGroupHeight(Track * t) const;
 
