@@ -32,12 +32,12 @@ default is to disable caching.
 
 * Read-caching: If caching is enabled, all block files will always be
   read-cached. Block files on disk will be read as soon as they are created
-  and hold in memory. New block files will be written to disk, but hold in
+  and held in memory. New block files will be written to disk, but held in
   memory, so they are never read from disk in the current session.
 
 * Write-caching: If caching is enabled and the parameter allowDeferredWrite
-  is enabled at the block file constructor, new block files are hold in memory
-  and only written to disk when WriteCacheToDisk() is called. This is used
+  is enabled at the block file constructor, new block files are held in memory
+  and written to disk only when WriteCacheToDisk() is called. This is used
   during recording to prevent disk access. After recording, WriteCacheToDisk()
   will be called on all block files and they will be written to disk. During
   normal editing, no write cache is active, that is, any block files will be
@@ -74,10 +74,6 @@ to get its definition, rather than rolling our own.
 #include "sndfile.h"
 #include "../Internat.h"
 
-//#define DEBUG_OUTPUT(s) printf("[SimpleBlockFile] %s\n", s)
-#define DEBUG_OUTPUT(s)
-
-
 
 wxUint32 SwapUintEndianess(wxUint32 in)
 {
@@ -104,20 +100,22 @@ wxUint32 SwapUintEndianess(wxUint32 in)
 SimpleBlockFile::SimpleBlockFile(wxFileName baseFileName,
                                  samplePtr sampleData, sampleCount sampleLen,
                                  sampleFormat format,
-                                 bool allowDeferredWrite /* = false */):
+                                 bool allowDeferredWrite /* = false */,
+                                 bool bypassCache /* = false */):
    BlockFile(wxFileName(baseFileName.GetFullPath() + wxT(".au")), sampleLen)
 {
    mCache.active = false;
    
-   DEBUG_OUTPUT("SimpleBlockFile created based on sample data");
+   bool useCache = GetCache() && (!bypassCache);
 
-   bool useCache = GetCache();
-
-   if (!(allowDeferredWrite && useCache))
-      WriteSimpleBlockFile(sampleData, sampleLen, format, NULL);
+   if (!(allowDeferredWrite && useCache) && !bypassCache)
+   {
+      bool bSuccess = WriteSimpleBlockFile(sampleData, sampleLen, format, NULL);
+      wxASSERT(bSuccess); // TODO: Handle failure here by alert to user and undo partial op. 
+   }
       
    if (useCache) {
-      DEBUG_OUTPUT("Caching block file data");
+      //wxLogDebug("SimpleBlockFile::SimpleBlockFile(): Caching block file data.");
       mCache.active = true;
       mCache.needWrite = true;
       mCache.format = format;
@@ -140,8 +138,6 @@ SimpleBlockFile::SimpleBlockFile(wxFileName existingFile, sampleCount len,
                                  float min, float max, float rms):
    BlockFile(existingFile, len)
 {
-   DEBUG_OUTPUT("SimpleBlockFile based on existing file created");
-   
    mMin = min;
    mMax = max;
    mRMS = rms;
@@ -164,14 +160,7 @@ bool SimpleBlockFile::WriteSimpleBlockFile(
     sampleFormat format,
     void* summaryData)
 {
-   // Now checked in the DirManager
-   //wxASSERT( !wxFileExists(FILENAME(mFileName.GetFullPath())) );
-   
-   DEBUG_OUTPUT("Writing simple block file");
-
-   // Open and write the file
    wxFFile file(mFileName.GetFullPath(), wxT("wb"));
-
    if( !file.IsOpened() ){
       // Can't do anything else.
       return false;
@@ -217,8 +206,21 @@ bool SimpleBlockFile::WriteSimpleBlockFile(
    if (!summaryData)
       summaryData = /*BlockFile::*/CalcSummary(sampleData, sampleLen, format); //mchinen:allowing virtual override of calc summary for ODDecodeBlockFile.
    
-   file.Write(&header, sizeof(header));
-   file.Write(summaryData, mSummaryInfo.totalSummaryBytes);
+   size_t nBytesToWrite = sizeof(header);
+   size_t nBytesWritten = file.Write(&header, nBytesToWrite);
+   if (nBytesWritten != nBytesToWrite)
+   {
+      wxLogDebug(wxT("Wrote %d bytes, expected %d."), nBytesWritten, nBytesToWrite);
+      return false;
+   }
+
+   nBytesToWrite = mSummaryInfo.totalSummaryBytes;
+   nBytesWritten = file.Write(summaryData, nBytesToWrite);
+   if (nBytesWritten != nBytesToWrite)
+   {
+      wxLogDebug(wxT("Wrote %d bytes, expected %d."), nBytesWritten, nBytesToWrite);
+      return false;
+   }
 
    if( format == int24Sample )
    {
@@ -228,20 +230,33 @@ bool SimpleBlockFile::WriteSimpleBlockFile(
       int *int24sampleData = (int*)sampleData;
 
       for( int i = 0; i < sampleLen; i++ )
-#if wxBYTE_ORDER == wxBIG_ENDIAN
-         file.Write((char*)&int24sampleData[i] + 1, 3);
-#else
-         file.Write((char*)&int24sampleData[i], 3);
-#endif
+      {
+         nBytesToWrite = 3;
+         nBytesWritten = 
+            #if wxBYTE_ORDER == wxBIG_ENDIAN
+               file.Write((char*)&int24sampleData[i] + 1, nBytesToWrite);
+            #else
+               file.Write((char*)&int24sampleData[i], nBytesToWrite);
+            #endif
+         if (nBytesWritten != nBytesToWrite)
+         {
+            wxLogDebug(wxT("Wrote %d bytes, expected %d."), nBytesWritten, nBytesToWrite);
+            return false;
+         }
+      }
    }
    else
    {
       // for all other sample formats we can write straight from the buffer
       // to disk
-      file.Write(sampleData, sampleLen * SAMPLE_SIZE(format));
-    }
-    
-    DEBUG_OUTPUT("Wrote simple block file");
+      nBytesToWrite = sampleLen * SAMPLE_SIZE(format);
+      nBytesWritten = file.Write(sampleData, nBytesToWrite);
+      if (nBytesWritten != nBytesToWrite)
+      {
+         wxLogDebug(wxT("Wrote %d bytes, expected %d."), nBytesWritten, nBytesToWrite);
+         return false;
+      }
+   }
     
     return true;
 }
@@ -251,8 +266,6 @@ void SimpleBlockFile::FillCache()
    if (mCache.active)
       return; // cache is already filled
 
-   DEBUG_OUTPUT("Reading simple block file into cache");
-   
    // Check sample format
    wxFFile file(mFileName.GetFullPath(), wxT("rb"));
    if (!file.IsOpened())
@@ -310,7 +323,7 @@ void SimpleBlockFile::FillCache()
    mCache.active = true;
    mCache.needWrite = false;
    
-   DEBUG_OUTPUT("Succesfully read simple blockfile into cache");
+   //wxLogDebug("SimpleBlockFile::FillCache(): Succesfully read simple block file into cache.");
 }
 
 /// Read the summary section of the disk file.
@@ -321,12 +334,12 @@ bool SimpleBlockFile::ReadSummary(void *data)
 {
    if (mCache.active)
    {
-      DEBUG_OUTPUT("ReadSummary: Summary is already in cache");
+      //wxLogDebug("SimpleBlockFile::ReadSummary(): Summary is already in cache.");
       memcpy(data, mCache.summaryData, (size_t)mSummaryInfo.totalSummaryBytes);
       return true;
    } else
    {
-      DEBUG_OUTPUT("ReadSummary: Reading summary from disk");
+      //wxLogDebug("SimpleBlockFile::ReadSummary(): Reading summary from disk.");
       
       wxFFile file(mFileName.GetFullPath(), wxT("rb"));
 
@@ -371,7 +384,7 @@ int SimpleBlockFile::ReadData(samplePtr data, sampleFormat format,
 {
    if (mCache.active)
    {
-      DEBUG_OUTPUT("ReadData: Data is already in cache");
+      //wxLogDebug("SimpleBlockFile::ReadData(): Data are already in cache.");
       
       if (len > mLen - start)
          len = mLen - start;
@@ -382,7 +395,7 @@ int SimpleBlockFile::ReadData(samplePtr data, sampleFormat format,
       return len;
    } else
    {
-      DEBUG_OUTPUT("ReadData: Reading data from disk");
+      //wxLogDebug("SimpleBlockFile::ReadData(): Reading data from disk.");
       
       SF_INFO info;
       wxLogNull *silence=0;
@@ -467,11 +480,14 @@ void SimpleBlockFile::SaveXML(XMLWriter &xmlFile)
    xmlFile.EndTag(wxT("simpleblockfile"));
 }
 
+// BuildFromXML methods should always return a BlockFile, not NULL,  
+// even if the result is flawed (e.g., refers to nonexistent file), 
+// as testing will be done in DirManager::ProjectFSCK().
 /// static
 BlockFile *SimpleBlockFile::BuildFromXML(DirManager &dm, const wxChar **attrs)
 {
    wxFileName fileName;
-   float min=0, max=0, rms=0;
+   float min = 0.0f, max = 0.0f, rms = 0.0f;
    sampleCount len = 0;
    double dblValue;
    long nValue;
@@ -480,41 +496,34 @@ BlockFile *SimpleBlockFile::BuildFromXML(DirManager &dm, const wxChar **attrs)
    {
       const wxChar *attr =  *attrs++;
       const wxChar *value = *attrs++;
-
       if (!value)
          break;
 
       const wxString strValue = value;
-      if( !wxStrcmp(attr, wxT("filename")) )
+      if (!wxStricmp(attr, wxT("filename")) && 
+            // Can't use XMLValueChecker::IsGoodFileName here, but do part of its test.
+            XMLValueChecker::IsGoodFileString(strValue) && 
+            (strValue.Length() + 1 + dm.GetProjectDataDir().Length() <= PLATFORM_MAX_PATH))
       {
-         // Can't use XMLValueChecker::IsGoodFileName here, but do part of its test.
-         if (!XMLValueChecker::IsGoodFileString(strValue))
-            return NULL;
-
-         #ifdef _WIN32
-            if (strValue.Length() + 1 + dm.GetProjectDataDir().Length() > MAX_PATH)
-               return NULL;
-         #endif
-
-         dm.AssignFile(fileName,value,FALSE);
+         if (!dm.AssignFile(fileName, strValue, false))
+            // Make sure fileName is back to uninitialized state so we can detect problem later.
+            fileName.Clear();
       }
-      else if( !wxStrcmp(attr, wxT("len")) && XMLValueChecker::IsGoodInt(strValue) && strValue.ToLong(&nValue)) 
+      else if (!wxStrcmp(attr, wxT("len")) && 
+               XMLValueChecker::IsGoodInt(strValue) && strValue.ToLong(&nValue) && 
+               nValue > 0) 
          len = nValue;
-      else if( !wxStrcmp(attr, wxT("min")) && 
-               XMLValueChecker::IsGoodString(strValue) && Internat::CompatibleToDouble(strValue, &dblValue))
-         min = dblValue;
-      else if( !wxStrcmp(attr, wxT("max")) && 
-               XMLValueChecker::IsGoodString(strValue) && Internat::CompatibleToDouble(strValue, &dblValue))
-         max = dblValue;
-      else if( !wxStrcmp(attr, wxT("rms")) && 
-               XMLValueChecker::IsGoodString(strValue) && Internat::CompatibleToDouble(strValue, &dblValue))
-         rms = dblValue;
+      else if (XMLValueChecker::IsGoodString(strValue) && Internat::CompatibleToDouble(strValue, &dblValue))
+      {  // double parameters
+         if (!wxStricmp(attr, wxT("min")))
+            min = dblValue;
+         else if (!wxStricmp(attr, wxT("max")))
+            max = dblValue;
+         else if (!wxStricmp(attr, wxT("rms")) && (dblValue >= 0.0))
+            rms = dblValue;
+      }
    }
 
-   if (!XMLValueChecker::IsGoodFileString(fileName.GetFullName()) || 
-         (len <= 0) || (rms < 0.0))
-      return NULL;
-
    return new SimpleBlockFile(fileName, len, min, max, rms);
 }
 
@@ -575,8 +584,6 @@ void SimpleBlockFile::WriteCacheToDisk()
    if (!GetNeedWriteCacheToDisk())
       return;
    
-   DEBUG_OUTPUT("WriteCacheToDisk");
-
    if (WriteSimpleBlockFile(mCache.sampleData, mLen, mCache.format,
                             mCache.summaryData))
       mCache.needWrite = false;
@@ -589,16 +596,22 @@ bool SimpleBlockFile::GetNeedWriteCacheToDisk()
 
 bool SimpleBlockFile::GetCache()
 {  
+#ifdef DEPRECATED_AUDIO_CACHE 
+   // See http://bugzilla.audacityteam.org/show_bug.cgi?id=545.
    bool cacheBlockFiles = false;
    gPrefs->Read(wxT("/Directories/CacheBlockFiles"), &cacheBlockFiles);
+   if (!cacheBlockFiles) 
+      return false;
 
    int lowMem = gPrefs->Read(wxT("/Directories/CacheLowMem"), 16l);
    if (lowMem < 16) {
       lowMem = 16;
    }
    lowMem <<= 20;
-    
-   return cacheBlockFiles && (wxGetFreeMemory() > lowMem);
+   return (GetFreeMemory() > lowMem);
+#else
+   return false;
+#endif
 }
 
 // Indentation settings for Vim and Emacs and unique identifier for Arch, a
