@@ -143,7 +143,7 @@ bool EffectCompressor::PromptUser()
    gPrefs->Write(wxT("/Effects/Compressor/Normalize"), mNormalize);
    gPrefs->Write(wxT("/Effects/Compressor/UsePeak"), mUsePeak);
 
-   return true;
+   return gPrefs->Flush();
 }
 
 bool EffectCompressor::NewTrackPass1()
@@ -532,12 +532,12 @@ void CompressorPanel::OnPaint(wxPaintEvent & evt)
    int startY = (int)rint((threshold*(1.0/ratio-1.0))*mEnvRect.height/rangeDB);
 
    // Yellow line for threshold
-   memDC.SetPen(wxPen(wxColour(220, 220, 0), 1, wxSOLID));
+/*   memDC.SetPen(wxPen(wxColour(220, 220, 0), 1, wxSOLID));
    AColor::Line(memDC,
                 mEnvRect.x,
                 mEnvRect.y + mEnvRect.height - kneeY,
                 mEnvRect.x + mEnvRect.width - 1,
-                mEnvRect.y + mEnvRect.height - kneeY);
+                mEnvRect.y + mEnvRect.height - kneeY);*/
 
    // Was: Nice dark red line for the compression diagram
 //   memDC.SetPen(wxPen(wxColour(180, 40, 40), 3, wxSOLID));
@@ -653,6 +653,9 @@ void CompressorDialog::PopulateOrExchange(ShuttleGui & S)
          mRatioText = S.AddVariableText(wxT("XXXX:1"), true,
                                              wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
 
+         /* i18n-hint: Particularly in percussion, sounds can be regarded as having 
+          * an 'attack' phase where the sound builds up and a 'decay' where the 
+          * sound dies away.  So this means 'onset duration'.  */
          mAttackLabel = S.AddVariableText(_("Attack Time:"), true,
                                          wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
          S.SetStyle(wxSL_HORIZONTAL);
@@ -675,6 +678,7 @@ void CompressorDialog::PopulateOrExchange(ShuttleGui & S)
 
    S.StartHorizontalLay(wxCENTER, false);
    {
+      /* i18n-hint: Make-up, i.e. correct for any reduction, rather than fabricate it.*/
       mGainCheckBox = S.AddCheckBox(_("Make-up gain for 0dB after compressing"),
                                     wxT("true"));
       mPeakCheckBox = S.AddCheckBox(_("Compress based on Peaks"),
@@ -717,6 +721,7 @@ bool CompressorDialog::TransferDataFromWindow()
    mPanel->ratio = ratio;
 
    mThresholdLabel->SetName(wxString::Format(_("Threshold %d dB"), (int)threshold));
+   /* i18n-hint: usually leave this as is as dB doesn't get translated*/
    mThresholdText->SetLabel(wxString::Format(_("%3d dB"), (int)threshold));
 
    mNoiseFloorLabel->SetName(wxString::Format(_("Noise Floor %d dB"), (int)noisefloor));
@@ -724,10 +729,14 @@ bool CompressorDialog::TransferDataFromWindow()
 
    if (mRatioSlider->GetValue()%2 == 0) {
       mRatioLabel->SetName(wxString::Format(_("Ratio %.0f to 1"), ratio));
+      /* i18n-hint: Unless your language has a different convention for ratios, 
+       * like 8:1, leave as is.*/
       mRatioText->SetLabel(wxString::Format(_("%.0f:1"), ratio));
    }
    else {
       mRatioLabel->SetName(wxString::Format(_("Ratio %.1f to 1"), ratio));
+      /* i18n-hint: Unless your language has a different convention for ratios, 
+       * like 8:1, leave as is.*/
       mRatioText->SetLabel(wxString::Format(_("%.1f:1"), ratio));
    }
 
