@@ -34,6 +34,7 @@
 #include <wx/utils.h>
 
 #if defined(__WXMAC__)
+#include <dlfcn.h>
 #include <wx/mac/private.h>
 #else
 #include <wx/dynlib.h>
@@ -75,6 +76,7 @@ void RegisterVSTEffects()
    if (gPrefs->Read(wxT("/VST/Rescan"), (long) false) != false) {
       pm.PurgeType(VSTPLUGINTYPE);
       gPrefs->Write(wxT("/VST/Rescan"), false);
+      gPrefs->Flush();
    }
 
    if (!pm.HasType(VSTPLUGINTYPE)) {
@@ -87,8 +89,14 @@ void RegisterVSTEffects()
 
    wxString path = pm.GetFirstPlugin(VSTPLUGINTYPE);
    while (!path.IsEmpty()) {
-      em.RegisterEffect(new VSTEffect(path));
-
+#if defined(__WXMAC__)
+      if (wxDirExists(path)) {
+#else
+      if (wxFileExists(path)) {
+#endif
+         em.RegisterEffect(new VSTEffect(path));
+      }
+      
       path = pm.GetNextPlugin(VSTPLUGINTYPE);
    }
 
@@ -110,6 +118,8 @@ class VSTEffectDialog:public wxDialog, XMLTagHandler
                    AEffect *aeffect);
    virtual ~VSTEffectDialog();
 
+   void RemoveHandler();
+
    void OnIdle(wxIdleEvent & evt);
 
    void OnProgram(wxCommandEvent & evt);
@@ -121,6 +131,7 @@ class VSTEffectDialog:public wxDialog, XMLTagHandler
 
    void OnOk(wxCommandEvent & evt);
    void OnCancel(wxCommandEvent & evt);
+   void OnClose(wxCloseEvent & evt);
    void OnPreview(wxCommandEvent & evt);
 
  private:
@@ -151,6 +162,11 @@ class VSTEffectDialog:public wxDialog, XMLTagHandler
    bool mInChunk;
    wxString mChunk;
 
+#if defined(__WXMAC__)
+   EventHandlerUPP mHandlerUPP;
+   EventHandlerRef mHandlerRef;
+#endif
+
    DECLARE_EVENT_TABLE()
 };
 
@@ -176,6 +192,31 @@ BEGIN_EVENT_TABLE(VSTEffectDialog, wxDialog)
    EVT_SLIDER(wxID_ANY, VSTEffectDialog::OnSlider)
 END_EVENT_TABLE()
 
+#if defined(__WXMAC__)
+
+// Event handler to capture the window close event
+static const EventTypeSpec eventList[] =
+{
+   {kEventClassWindow, kEventWindowClose},
+};
+
+static pascal OSStatus EventHandler(EventHandlerCallRef handler, EventRef event, void *data)
+{
+   OSStatus result = eventNotHandledErr;
+
+   VSTEffectDialog *dlg = (VSTEffectDialog *)data;
+
+   if (GetEventClass(event) == kEventClassWindow && GetEventKind(event) == kEventWindowClose) {
+      dlg->RemoveHandler();
+      dlg->Close();
+      result = noErr;
+   }
+
+   return result;
+}
+
+#endif
+
 VSTEffectDialog::VSTEffectDialog(wxWindow *parent,
                                        const wxString & title,
                                        VSTEffect *effect,
@@ -188,6 +229,10 @@ VSTEffectDialog::VSTEffectDialog(wxWindow *parent,
    mSliders = NULL;
    mDisplays = NULL;
    mLabels = NULL;
+#if defined(__WXMAC__)
+   mHandlerUPP = NULL;
+   mHandlerRef = NULL;
+#endif
 
    // Determine if the VST editor is supposed to be used or not
    mGui = (gPrefs->Read(wxT("/VST/GUI"), (long) true) != 0) &&
@@ -204,6 +249,8 @@ VSTEffectDialog::VSTEffectDialog(wxWindow *parent,
 
 VSTEffectDialog::~VSTEffectDialog()
 {
+   RemoveHandler();
+   
    if (mNames) {
       delete [] mNames;	
    }
@@ -221,6 +268,22 @@ VSTEffectDialog::~VSTEffectDialog()
    }
 }
 
+void VSTEffectDialog::RemoveHandler()
+{
+#if defined(__WXMAC__)
+   if (mHandlerRef) {
+      ::RemoveEventHandler(mHandlerRef);
+      mHandlerRef = NULL;
+      MacInstallTopLevelWindowEventHandler();
+   }
+
+   if (mHandlerUPP) {
+      DisposeEventHandlerUPP(mHandlerUPP);
+      mHandlerUPP = NULL;
+   }
+#endif
+}
+
 void VSTEffectDialog::BuildFancy()
 {
    struct
@@ -272,9 +335,22 @@ void VSTEffectDialog::BuildFancy()
 #if defined(__WXMAC__)
    HIViewPlaceInSuperviewAt(subview, pos.x, pos.y);
 
-   // LL:  Some VST effects do not work unless this is done.  But, it must be
-   //      done last since proper window sizing will not occur otherwise.
+   // Some VST effects do not work unless the default handler is removed since
+   // it captures many of the events that the plugins need.  But, it must be
+   // done last since proper window sizing will not occur otherwise.
    ::RemoveEventHandler((EventHandlerRef)MacGetEventHandler());
+
+   // Install a bare minimum handler so we can capture the window close event.  If
+   // it's not captured, we will crash at Audacity termination since the window
+   // is still on the wxWidgets toplevel window lists, but it's already gone.
+   mHandlerUPP = NewEventHandlerUPP(EventHandler);
+   InstallWindowEventHandler(win,
+                             mHandlerUPP,
+                             GetEventTypeCount(eventList),
+                             eventList,
+                             this,
+                             &mHandlerRef);
+
 #elif defined(__WXMSW__)
    w->SetPosition(pos);
    w->SetSize(si->GetSize());
@@ -411,6 +487,7 @@ wxSizer *VSTEffectDialog::BuildProgramBar()
                              wxSize(200, -1),
                              progs
                              );
+   mProgram->SetName(_("Presets"));
    hs->Add(mProgram, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
 
    wxButton *bt = new wxButton(this, ID_VST_LOAD, _("Load"));
@@ -534,7 +611,7 @@ void VSTEffectDialog::OnLoad(wxCommandEvent & evt)
    if (!reader.Parse(this, fn)) {
       // Inform user of load failure
       wxMessageBox(reader.GetErrorStr(),
-                   _("Error loading program"),
+                   _("Error Loading VST Program"),
                    wxOK | wxCENTRE,
                    this);
    }
@@ -616,6 +693,11 @@ void VSTEffectDialog::OnSave(wxCommandEvent & evt)
    xmlFile.Close();
 }
 
+void VSTEffectDialog::OnClose(wxCloseEvent & evt)
+{
+   EndModal(false);
+}
+
 void VSTEffectDialog::OnPreview(wxCommandEvent & evt)
 {
    mEffect->Preview();
@@ -1191,7 +1273,10 @@ bool VSTEffect::PromptUser()
    dlog.CentreOnParent();
    dlog.ShowModal();
 
-   return dlog.GetReturnCode() != 0;
+   bool ret = dlog.GetReturnCode() != 0;
+
+   dlog.Destroy();
+   return ret;
 }
 
 bool VSTEffect::Process()
@@ -1226,7 +1311,7 @@ bool VSTEffect::Process()
       }
 
       if (mBlockSize == 0) {
-         mBlockSize = left->GetMaxBlockSize() * 2;
+         mBlockSize = mWTBlockSize = left->GetMaxBlockSize() * 2;
 
          // Some VST effects (Antress Modern is an example), do not like
          // overly large block sizes.  Unfortunately, I have not found a
@@ -1243,9 +1328,11 @@ bool VSTEffect::Process()
             mInBuffer[i] = new float[mBlockSize];
          }
 
+         //Process 2 audacity blockfiles per WaveTrack::Set independently of mBlockSize
+         //because it is extremely slow to do multiple Set()s per blockfile.
          mOutBuffer = new float *[mOutputs];
          for (int i = 0; i < mOutputs; i++) {
-            mOutBuffer[i] = new float[mBlockSize];
+            mOutBuffer[i] = new float[mWTBlockSize + mBlockSize];
          }
 
          // Turn the power off
@@ -1301,6 +1388,7 @@ bool VSTEffect::ProcessStereo(int count,
                               sampleCount len)
 {
    bool rc = true;
+   sampleCount amountLeft = 0;
 
    // Initialize time info
    mTimeInfo.samplePos = 0.0;
@@ -1317,6 +1405,10 @@ bool VSTEffect::ProcessStereo(int count,
    sampleCount originalLen = len;
    sampleCount ls = lstart;
    sampleCount rs = rstart;
+   sampleCount outls = lstart;
+   sampleCount outrs = rstart;
+   sampleCount outBufferCursor = 0;
+   float **outBufSegment = new float *[mOutputs];
    while (len) {
       int block = mBlockSize;
       if (block > len) {
@@ -1328,12 +1420,27 @@ bool VSTEffect::ProcessStereo(int count,
          right->Get((samplePtr)mInBuffer[1], floatSample, rs, block);
       }
 
-      callProcessReplacing(mInBuffer, mOutBuffer, block);
-
-      left->Set((samplePtr)mOutBuffer[0], floatSample, ls, block);
-      if (right) {
-         right->Set((samplePtr)mOutBuffer[1], floatSample, rs, block);
-      }      
+      for (int i = 0; i < mOutputs; i++)
+         outBufSegment[i] = mOutBuffer[i ] + outBufferCursor;
+      callProcessReplacing(mInBuffer, outBufSegment, block);
+      outBufferCursor += block;
+      //Process 2 audacity blockfiles per WaveTrack::Set independently of mBlockSize
+      //because it is extremely slow to do multiple Set()s per blockfile due to Undo History
+      //If we do more optimization we should probably align the Sets to blockfile boundries.
+      if (outBufferCursor >= mWTBlockSize) {
+         left->Set((samplePtr)mOutBuffer[0], floatSample, outls, mWTBlockSize);
+         if (right) {
+            right->Set((samplePtr)mOutBuffer[1], floatSample, outrs, mWTBlockSize);
+         }
+         if (outBufferCursor >= mWTBlockSize) {
+            //snake the buffer down
+            memmove(mOutBuffer[0], mOutBuffer[0] + mWTBlockSize, SAMPLE_SIZE(floatSample) * (outBufferCursor - mWTBlockSize));
+            memmove(mOutBuffer[1], mOutBuffer[1] + mWTBlockSize, SAMPLE_SIZE(floatSample) * (outBufferCursor - mWTBlockSize));
+         }
+         outBufferCursor -= mWTBlockSize;
+         outls += mWTBlockSize;
+         outrs += mWTBlockSize;
+      }
 
       len -= block;
       ls += block;
@@ -1354,6 +1461,14 @@ bool VSTEffect::ProcessStereo(int count,
       }
    }
 
+   //finish taking the remainder.
+   if (outBufferCursor) {
+     left->Set((samplePtr)mOutBuffer[0], floatSample, outls, outBufferCursor);
+     if (right) {
+         right->Set((samplePtr)mOutBuffer[1], floatSample, outrs, outBufferCursor);
+      }
+   }
+
    // Tell effect we're done
    callDispatcher(effStopProcess, 0, 0, NULL, 0.0);
 
@@ -1384,6 +1499,9 @@ bool VSTEffect::Load()
    mAEffect = NULL;
 
 #if defined(__WXMAC__)
+   // Start clean
+   mBundleRef = NULL;
+
    // Don't really know what this should be initialize to
    mResource = -1;
 
@@ -1411,31 +1529,58 @@ bool VSTEffect::Load()
       return false;
    }
 
-   // Try to locate the new plugin entry point
-   pluginMain = (vstPluginMain)
-      CFBundleGetFunctionPointerForName(bundleRef,
-                                        CFSTR("VSTPluginMain"));
-
-   // If not found, try finding the old entry point
-   if (pluginMain == NULL) {
-      pluginMain = (vstPluginMain)
-         CFBundleGetFunctionPointerForName(bundleRef,
-                                           CFSTR("main_macho"));
-   }
-
-   // Must not be a VST plugin
-   if (pluginMain == NULL) {
+   // Retrieve a reference to the executable
+   CFURLRef exeRef = CFBundleCopyExecutableURL(bundleRef);
+   if (exeRef == NULL) {
       CFRelease(bundleRef);
       return false;
    }
 
+   // Convert back to path
+   UInt8 exePath[PATH_MAX];
+   success = CFURLGetFileSystemRepresentation(exeRef, true, exePath, sizeof(exePath));
+
+   // Done with the executable reference
+   CFRelease(exeRef);
+
+   // Bail if we couldn't resolve the executable path
+   if (success == FALSE) {
+      CFRelease(bundleRef);
+      return false;
+   }
+
+   // Attempt to open it
+   mModule = dlopen((char *) exePath, RTLD_NOW | RTLD_LOCAL);
+   if (mModule == NULL) {
+      CFRelease(bundleRef);
+      return false;
+   }
+
+   // Try to locate the new plugin entry point
+   pluginMain = (vstPluginMain) dlsym(mModule, "VSTPluginMain");
+
+   // If not found, try finding the old entry point
+   if (pluginMain == NULL) {
+      pluginMain = (vstPluginMain) dlsym(mModule, "main_macho");
+   }
+
+   // Must not be a VST plugin
+   if (pluginMain == NULL) {
+      dlclose(mModule);
+      mModule = NULL;
+      CFRelease(bundleRef);
+      return false;
+   }
+
+   // Need to keep the bundle reference around so we can map the
+   // resources.
+   mBundleRef = bundleRef;
+
    // Open the resource map ... some plugins (like GRM Tools) need this.
    mResource = (int) CFBundleOpenBundleResourceMap(bundleRef);
 
-   // Save the bundle reference
-   mModule = bundleRef;
-
 #else
+
    {
       wxLogNull nolog;
 
@@ -1511,10 +1656,16 @@ void VSTEffect::Unload()
    if (mModule) {
 #if defined(__WXMAC__)
       if (mResource != -1) {
-         CFBundleCloseBundleResourceMap((CFBundleRef) mModule, mResource);
+         CFBundleCloseBundleResourceMap((CFBundleRef) mBundleRef, mResource);
+         mResource = -1;
       }
 
-      CFRelease((CFBundleRef) mModule);
+      if (mBundleRef != NULL) {
+         CFRelease((CFBundleRef) mBundleRef);
+         mBundleRef = NULL;
+      }
+
+      dlclose(mModule);
 #else
       delete (wxDynamicLibrary *) mModule;
 #endif
@@ -1568,28 +1719,47 @@ void VSTEffect::Scan()
 #elif defined(__WXMSW__)
    TCHAR dpath[MAX_PATH];
    TCHAR tpath[MAX_PATH];
-   DWORD len = WXSIZEOF(tpath);
+   DWORD len;
 
-   // Setup the default VST path.
-   dpath[0] = '\0';
-   ExpandEnvironmentStrings(wxT("%ProgramFiles%\\Steinberg\\VSTPlugins"),
-                            dpath,
-                            WXSIZEOF(dpath));
-
-   // Check registry for the real path
+   // Try HKEY_CURRENT_USER registry key first
+   len = sizeof(tpath) / sizeof(TCHAR);
    if (SHRegGetUSValue(wxT("Software\\VST"),
-                          wxT("VSTPluginsPath"),
-                          NULL,
-                          tpath,
-                          &len,
-                          FALSE,
-                          dpath,
-                          (DWORD) _tcslen(dpath)) == ERROR_SUCCESS) {
+                       wxT("VSTPluginsPath"),
+                       NULL,
+                       tpath,
+                       &len,
+                       FALSE,
+                       NULL,
+                       0) == ERROR_SUCCESS) {
       tpath[len] = 0;
+      dpath[0] = 0;
       ExpandEnvironmentStrings(tpath, dpath, WXSIZEOF(dpath));
       wxGetApp().AddUniquePathToPathList(LAT1CTOWX(dpath), pathList);
    }
 
+   // Then try HKEY_LOCAL_MACHINE registry key
+   len = sizeof(tpath) / sizeof(TCHAR);
+   if (SHRegGetUSValue(wxT("Software\\VST"),
+                       wxT("VSTPluginsPath"),
+                       NULL,
+                       tpath,
+                       &len,
+                       TRUE,
+                       NULL,
+                       0) == ERROR_SUCCESS) {
+      tpath[len] = 0;
+      dpath[0] = 0;
+      ExpandEnvironmentStrings(tpath, dpath, WXSIZEOF(dpath));
+      wxGetApp().AddUniquePathToPathList(LAT1CTOWX(dpath), pathList);
+   }
+
+   // Add the default path last
+   dpath[0] = 0;
+   ExpandEnvironmentStrings(wxT("%ProgramFiles%\\Steinberg\\VSTPlugins"),
+                            dpath,
+                            WXSIZEOF(dpath));
+   wxGetApp().AddUniquePathToPathList(LAT1CTOWX(dpath), pathList);
+
    // Recursively scan for all DLLs
    wxGetApp().FindFilesInPathList(wxT("*.dll"), pathList, files, wxDIR_DEFAULT);
 
@@ -1606,6 +1776,11 @@ void VSTEffect::Scan()
    size_t cnt = files.GetCount();
    wxString longest;
 
+   // JKC: Let's not show the progress dialog if there are no 
+   // files to test.
+   if( cnt <= 0 )
+      return;
+
    for (size_t i = 0; i < cnt; i++) {
       if (files[i].Length() > longest.Length()) {
          longest = files[i];
@@ -1634,6 +1809,8 @@ void VSTEffect::Scan()
       }
 
       argv[2] = file.c_str();
+      // ToDo: do we need a try--catch around this in case a bad plug-in 
+      // fails? (JKC Nov09)
       wxExecute((wxChar **) argv, wxEXEC_SYNC | wxEXEC_NODISABLE, NULL);
    }
 
