@@ -23,8 +23,9 @@ Licensed under the GNU General Public License v2 or later
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
 
-#include "../Audacity.h"	// needed before FFmpeg.h
-#include "../FFmpeg.h"		// which brings in avcodec.h, avformat.h
+#include "../Audacity.h"    // needed before FFmpeg.h
+#include "../FFmpeg.h"      // which brings in avcodec.h, avformat.h
+#include "../ondemand/ODManager.h"
 #ifndef WX_PRECOMP
 // Include your minimal set of headers here, or wx.h
 #include <wx/window.h>
@@ -153,31 +154,15 @@ static const wxChar *exts[] =
 #include "../WaveTrack.h"
 #include "ImportPlugin.h"
 
+
+#ifdef EXPERIMENTAL_OD_FFMPEG
+#include "../ondemand/ODDecodeFFmpegTask.h"
+#endif
+
 extern FFmpegLibs *FFmpegLibsInst;
 
 class FFmpegImportFileHandle;
 
-typedef struct _streamContext
-{
-   bool                 m_use;                           // TRUE = this stream will be loaded into Audacity
-   AVStream            *m_stream;		      	         // an AVStream *
-   AVCodecContext      *m_codecCtx;			               // pointer to m_stream->codec
-
-   AVPacket             m_pkt;				               // the last AVPacket we read for this stream
-   int                  m_pktValid;			               // is m_pkt valid?
-   uint8_t             *m_pktDataPtr;		           	   // pointer into m_pkt.data
-   int                  m_pktRemainingSiz;	
-
-   int64_t			      m_pts;			              	   // the current presentation time of the input stream
-   int64_t			      m_ptsOffset;		    	         // packets associated with stream are relative to this
-
-   int                  m_frameValid;		               // is m_decodedVideoFrame/m_decodedAudioSamples valid?
-   int16_t             *m_decodedAudioSamples;           // decoded audio samples stored here
-   unsigned int			m_decodedAudioSamplesSiz;        // current size of m_decodedAudioSamples
-   int			         m_decodedAudioSamplesValidSiz;   // # valid bytes in m_decodedAudioSamples
-   int                  m_initialchannels;               // number of channels allocated when we begin the importing. Assumes that number of channels doesn't change on the fly.
-} streamContext;
-
 /// A representative of FFmpeg loader in
 /// the Audacity import plugin list
 class FFmpegImportPlugin : public ImportPlugin
@@ -191,6 +176,7 @@ public:
 
    ~FFmpegImportPlugin() { }
 
+   wxString GetPluginStringID() { return wxT("libav"); }
    wxString GetPluginFormatDescription();
 
    ///! Probes the file and opens it if appropriate
@@ -239,8 +225,15 @@ public:
    ///! Writes extracted metadata to tags object
    ///\param avf - file context
    ///\ tags - Audacity tags object
-   void WriteMetadata(AVFormatContext *avf, Tags *tags);
+   void WriteMetadata(Tags *tags);
    
+   ///! Retrieves metadata from FFmpeg and converts to wxString
+   ///\param avf - file context
+   ///\ tags - Audacity tags object
+   ///\ tag - name of tag to set
+   ///\ name - name of metadata item to retrieve
+   void GetMetadata(Tags *tags, const wxChar *tag, const char *name);
+
    ///! Called by Import.cpp
    ///\return number of readable streams in the file
    wxInt32 GetStreamCount()
@@ -278,6 +271,10 @@ private:
    bool                  mStopped;       //!< True if importing was stopped by user
    wxString              mName;
    WaveTrack           ***mChannels;     //!< 2-dimentional array of WaveTrack's. First dimention - streams, second - channels of a stream. Length is mNumStreams
+#ifdef EXPERIMENTAL_OD_FFMPEG
+   bool                  mUsingOD;
+#endif
+
 };
 
 
@@ -316,6 +313,7 @@ ImportFileHandle *FFmpegImportPlugin::Open(wxString filename)
          if (dontShowDlg == 0 && newsession)
          {
             gPrefs->Write(wxT("/NewImportingSession"), false);
+            gPrefs->Flush();
             dlg = new FFmpegNotFoundDialog(NULL);
             dlg->ShowModal();
             delete dlg;
@@ -362,19 +360,19 @@ bool FFmpegImportFileHandle::Init()
 
    if (!FFmpegLibsInst->ValidLibsLoaded()) return false;
 
-   FFmpegLibsInst->av_log_set_callback(av_log_wx_callback);
+   av_log_set_callback(av_log_wx_callback);
 
    int err = ufile_fopen_input(&mFormatContext, mName);
    if (err < 0)
    {
-      wxLogMessage(wxT("FFmpeg : av_open_input_file() failed for file %s"),mName.c_str());
+      wxLogError(wxT("FFmpeg : av_open_input_file() failed for file %s"),mName.c_str());
       return false;
    }
 
-   err = FFmpegLibsInst->av_find_stream_info(mFormatContext);
+   err = av_find_stream_info(mFormatContext);
    if (err < 0)
    {
-      wxLogMessage(wxT("FFmpeg : av_find_stream_info() failed for file %s"),mName.c_str());
+      wxLogError(wxT("FFmpeg : av_find_stream_info() failed for file %s"),mName.c_str());
       return false;
    }
 
@@ -399,25 +397,25 @@ bool FFmpegImportFileHandle::InitCodecs()
          sc->m_stream = mFormatContext->streams[i];
          sc->m_codecCtx = sc->m_stream->codec;
 
-         AVCodec *codec = FFmpegLibsInst->avcodec_find_decoder(sc->m_codecCtx->codec_id);
+         AVCodec *codec = avcodec_find_decoder(sc->m_codecCtx->codec_id);
          if (codec == NULL)
          {
-            wxLogMessage(wxT("FFmpeg : avcodec_find_decoder() failed. Index[%02d], Codec[%02x - %s]"),i,sc->m_codecCtx->codec_id,sc->m_codecCtx->codec_name);
+            wxLogError(wxT("FFmpeg : avcodec_find_decoder() failed. Index[%02d], Codec[%02x - %s]"),i,sc->m_codecCtx->codec_id,sc->m_codecCtx->codec_name);
             //FFmpeg can't decode this stream, skip it
             delete sc;
             continue;
          }
          if (codec->type != sc->m_codecCtx->codec_type)
          {
-            wxLogMessage(wxT("FFmpeg : Codec type mismatch, skipping. Index[%02d], Codec[%02x - %s]"),i,sc->m_codecCtx->codec_id,sc->m_codecCtx->codec_name);
+            wxLogError(wxT("FFmpeg : Codec type mismatch, skipping. Index[%02d], Codec[%02x - %s]"),i,sc->m_codecCtx->codec_id,sc->m_codecCtx->codec_name);
             //Non-audio codec reported as audio? Nevertheless, we don't need THIS.
             delete sc;
             continue;
          }
 
-         if (FFmpegLibsInst->avcodec_open(sc->m_codecCtx, codec) < 0)
+         if (avcodec_open(sc->m_codecCtx, codec) < 0)
          {
-            wxLogMessage(wxT("FFmpeg : avcodec_open() failed. Index[%02d], Codec[%02x - %s]"),i,sc->m_codecCtx->codec_id,sc->m_codecCtx->codec_name);
+            wxLogError(wxT("FFmpeg : avcodec_open() failed. Index[%02d], Codec[%02x - %s]"),i,sc->m_codecCtx->codec_id,sc->m_codecCtx->codec_name);
             //Can't open decoder - skip this stream
             delete sc;
             continue;
@@ -435,7 +433,14 @@ bool FFmpegImportFileHandle::InitCodecs()
            bitrate.Printf(wxT("%d"),sc->m_codecCtx->bit_rate);
          else
            bitrate.Printf(wxT("?"));
-         strinfo.Printf(_("Index[%02x] Codec[%S], Language[%S], Bitrate[%S], Channels[%d], Duration[%d]"),sc->m_stream->id,codec->name,sc->m_stream->language,bitrate.c_str(),sc->m_stream->codec->channels, duration);
+
+         AVMetadataTag *tag = av_metadata_get(sc->m_stream->metadata, "language", NULL, 0);
+         wxString lang;
+         if (tag)
+         {
+            lang.FromUTF8(tag->value);
+         }
+         strinfo.Printf(_("Index[%02x] Codec[%S], Language[%S], Bitrate[%S], Channels[%d], Duration[%d]"),sc->m_stream->id,codec->name,lang.c_str(),bitrate.c_str(),sc->m_stream->codec->channels, duration);
          mStreamInfo->Add(strinfo);
          mScs[mNumStreams++] = sc;
       }
@@ -457,7 +462,6 @@ int FFmpegImportFileHandle::GetFileUncompressedBytes()
    return 0;
 }
 
-
 int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
               Track ***outTracks,
               int *outNumTracks,
@@ -485,13 +489,26 @@ int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
 
    for (int s = 0; s < mNumStreams; s++)
    {
+      switch (mScs[s]->m_stream->codec->sample_fmt)
+      {
+         case SAMPLE_FMT_U8:
+         case SAMPLE_FMT_S16:
+            mScs[s]->m_osamplesize = sizeof(int16_t);
+            mScs[s]->m_osamplefmt = int16Sample;
+         break;
+         default:
+            mScs[s]->m_osamplesize = sizeof(float);
+            mScs[s]->m_osamplefmt = floatSample;
+         break;
+      }
+
       // There is a possibility that number of channels will change over time, but we do not have WaveTracks for new channels. Remember the number of channels and stick to it.
       mScs[s]->m_initialchannels = mScs[s]->m_stream->codec->channels;
       mChannels[s] = new WaveTrack *[mScs[s]->m_stream->codec->channels];
       int c;
       for (c = 0; c < mScs[s]->m_stream->codec->channels; c++)
       {
-         mChannels[s][c] = trackFactory->NewWaveTrack(int16Sample, mScs[s]->m_stream->codec->sample_rate);
+         mChannels[s][c] = trackFactory->NewWaveTrack(mScs[s]->m_osamplefmt, mScs[s]->m_stream->codec->sample_rate);
 
          if (mScs[s]->m_stream->codec->channels == 2)
          {
@@ -519,10 +536,10 @@ int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
    for (int s = 0; s < mNumStreams; s++)
    {
       int64_t stream_delay = 0;
-      if (mScs[s]->m_stream->start_time != int64_t(AV_NOPTS_VALUE))
+      if (mScs[s]->m_stream->start_time != int64_t(AV_NOPTS_VALUE) && mScs[s]->m_stream->start_time > 0)
       {
          stream_delay = mScs[s]->m_stream->start_time;
-         wxLogMessage(wxT("Stream %d start_time = %d, that would be %f milliseconds."), s, mScs[s]->m_stream->start_time, double(mScs[s]->m_stream->start_time)/AV_TIME_BASE*1000);
+         wxLogError(wxT("Stream %d start_time = %d, that would be %f milliseconds."), s, mScs[s]->m_stream->start_time, double(mScs[s]->m_stream->start_time)/AV_TIME_BASE*1000);
       }
       if (stream_delay != 0)
       {
@@ -533,11 +550,68 @@ int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
          }
       }
    }
-
    // This is the heart of the importing process
-   streamContext *sc = NULL;
    // The result of Import() to be returend. It will be something other than zero if user canceled or some error appears.
    int res = eProgressSuccess;
+
+#ifdef EXPERIMENTAL_OD_FFMPEG
+   mUsingOD = false;
+   gPrefs->Read(wxT("/Library/FFmpegOnDemand"), &mUsingOD);
+   //at this point we know the file is good and that we have to load the number of channels in mScs[s]->m_stream->codec->channels;
+   //so for OD loading we create the tracks and releasee the modal lock after starting the ODTask.
+   if (mUsingOD) {
+      std::vector<ODDecodeFFmpegTask*> tasks;
+      //append blockfiles to each stream and add an individual ODDecodeTask for each one.
+      for (int s = 0; s < mNumStreams; s++) {
+         ODDecodeFFmpegTask* odTask=new ODDecodeFFmpegTask(mScs,mNumStreams,mChannels,mFormatContext, s);
+         odTask->CreateFileDecoder(mFilename);
+
+         //each stream has different duration.  We need to know it if seeking is to be allowed.
+         sampleCount sampleDuration = 0;
+         if (mScs[s]->m_stream->duration > 0)
+            sampleDuration = ((sampleCount)mScs[s]->m_stream->duration * mScs[s]->m_stream->time_base.num) *mScs[s]->m_stream->codec->sample_rate / mScs[s]->m_stream->time_base.den;
+         else
+            sampleDuration = ((sampleCount)mFormatContext->duration *mScs[s]->m_stream->codec->sample_rate) / AV_TIME_BASE;
+
+         //      printf(" OD duration samples %qi, sr %d, secs %d\n",sampleDuration, (int)mScs[s]->m_stream->codec->sample_rate,(int)sampleDuration/mScs[s]->m_stream->codec->sample_rate);
+         
+         //for each wavetrack within the stream add coded blockfiles
+         for (int c = 0; c < mScs[s]->m_stream->codec->channels; c++) {
+            WaveTrack *t = mChannels[s][c];
+            odTask->AddWaveTrack(t);
+         
+            sampleCount maxBlockSize = t->GetMaxBlockSize();
+            //use the maximum blockfile size to divide the sections (about 11secs per blockfile at 44.1khz)
+            for (sampleCount i = 0; i < sampleDuration; i += maxBlockSize) {
+               sampleCount blockLen = maxBlockSize;
+               if (i + blockLen > sampleDuration)
+                  blockLen = sampleDuration - i;
+            
+               t->AppendCoded(mFilename, i, blockLen, c,ODTask::eODFFMPEG);
+            
+               // This only works well for single streams since we assume 
+               // each stream is of the same duration and channels
+               res = mProgress->Update(i+sampleDuration*c+ sampleDuration*mScs[s]->m_stream->codec->channels*s, 
+                                       sampleDuration*mScs[s]->m_stream->codec->channels*mNumStreams);
+               if (res != eProgressSuccess)
+                  break;
+            }
+         }
+         tasks.push_back(odTask);
+      }
+      //Now we add the tasks and let them run, or delete them if the user cancelled
+      for(int i=0; i < (int)tasks.size(); i++) {
+         if(res==eProgressSuccess)
+            ODManager::Instance()->AddNewTask(tasks[i]);
+         else
+            {
+               delete tasks[i];
+            }
+      }
+   } else {
+#endif
+   streamContext *sc = NULL;
+
    // Read next frame.
    while ((sc = ReadNextFrame()) != NULL && (res == eProgressSuccess))
    {
@@ -558,7 +632,7 @@ int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
          // Cleanup after frame decoding
          if (sc->m_pktValid)
          {
-            FFmpegLibsInst->av_free_packet(&sc->m_pkt);
+            av_free_packet(&sc->m_pkt);
             sc->m_pktValid = 0;
          }    
       }
@@ -575,12 +649,15 @@ int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
 
             if (mScs[i]->m_pktValid)
             {
-               FFmpegLibsInst->av_free_packet(&mScs[i]->m_pkt);
+               av_free_packet(&mScs[i]->m_pkt);
                mScs[i]->m_pktValid = 0;
-            }				
+            }               
          }
       }
    }
+#ifdef EXPERIMENTAL_OD_FFMPEG
+   } // else -- !mUsingOD == true
+#endif   //EXPERIMENTAL_OD_FFMPEG
 
    // Something bad happened - destroy everything!
    if (res == eProgressCancelled || res == eProgressFailed)
@@ -618,122 +695,23 @@ int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
    delete[] mChannels;
 
    // Save metadata
-   WriteMetadata(mFormatContext,tags);
+   WriteMetadata(tags);
 
    return res;
 }
 
 streamContext *FFmpegImportFileHandle::ReadNextFrame()
 {
-   streamContext *sc = NULL;
-   AVPacket pkt;
-
-   if (FFmpegLibsInst->av_read_frame(mFormatContext,&pkt) < 0)
-   {
-      return NULL;
-   }
-
-   // Find a stream to which this frame belongs to
-   for (int i = 0; i < mNumStreams; i++)
-   {
-      if (mScs[i]->m_stream->index == pkt.stream_index)
-         sc = mScs[i];
-   }
-
-   // Off-stream packet. Don't panic, just skip it.
-   // When not all streams are selected for import this will happen very often.
-   if (sc == NULL)
-   {
-      FFmpegLibsInst->av_free_packet(&pkt);
-      return (streamContext*)1;
-   }
-
-   // Copy the frame to the stream context
-   memcpy(&sc->m_pkt, &pkt, sizeof(AVPacket));
-
-   sc->m_pktValid = 1;
-   sc->m_pktDataPtr = pkt.data;
-   sc->m_pktRemainingSiz = pkt.size;
-
-   return sc;
+   return import_ffmpeg_read_next_frame(mFormatContext, mScs, mNumStreams);
 }
 
 int FFmpegImportFileHandle::DecodeFrame(streamContext *sc, bool flushing)
 {
-   int		nBytesDecoded;			
-   wxUint8 *pDecode = sc->m_pktDataPtr;
-   int		nDecodeSiz = sc->m_pktRemainingSiz;
-
-   sc->m_frameValid = 0;
-
-   if (flushing)
-   {
-      // If we're flushing the decoders we don't actually have any new data to decode.
-      pDecode = NULL;
-      nDecodeSiz = 0;
-   }
-   else
-   {
-      if (!sc->m_pktValid || (sc->m_pktRemainingSiz <= 0))
-      {
-         //No more data
-         return -1;
-      }
-   }
-
-   // Reallocate the audio sample buffer if it's smaller than the frame size.
-   if (!flushing)
-   {
-      // av_fast_realloc() will only reallocate the buffer if m_decodedAudioSamplesSiz is 
-      // smaller than third parameter. It also returns new size in m_decodedAudioSamplesSiz
-      //\warning { for some reason using the following macro call right in the function call
-      // causes Audacity to crash in some unknown place. With "newsize" it works fine }
-      int newsize = FFMAX(sc->m_pkt.size*sizeof(*sc->m_decodedAudioSamples), AVCODEC_MAX_AUDIO_FRAME_SIZE);
-      sc->m_decodedAudioSamples = (int16_t*)FFmpegLibsInst->av_fast_realloc(sc->m_decodedAudioSamples, 
-         &sc->m_decodedAudioSamplesSiz,
-         newsize
-         );
-
-      if (sc->m_decodedAudioSamples == NULL)
-      {
-         //Can't allocate bytes
-         return -1;
-      }
-   }
-
-   // avcodec_decode_audio2() expects the size of the output buffer as the 3rd parameter but
-   // also returns the number of bytes it decoded in the same parameter.
-   sc->m_decodedAudioSamplesValidSiz = sc->m_decodedAudioSamplesSiz;
-   nBytesDecoded = FFmpegLibsInst->avcodec_decode_audio2(sc->m_codecCtx, 
-      sc->m_decodedAudioSamples,		      // out
-      &sc->m_decodedAudioSamplesValidSiz,	// in/out
-      pDecode, nDecodeSiz);				   // in
-
-   if (nBytesDecoded < 0)
-   {
-      // Decoding failed. Don't stop.
-      return -1;
-   }
-
-   // We may not have read all of the data from this packet. If so, the user can call again.
-   // Whether or not they do depends on if m_pktRemainingSiz == 0 (they can check).
-   sc->m_pktDataPtr += nBytesDecoded;
-   sc->m_pktRemainingSiz -= nBytesDecoded;
-
-   // At this point it's normally safe to assume that we've read some samples. However, the MPEG
-   // audio decoder is broken. If this is the case then we just return with m_frameValid == 0
-   // but m_pktRemainingSiz perhaps != 0, so the user can call again.
-   if (sc->m_decodedAudioSamplesValidSiz > 0)
-   {
-      sc->m_frameValid = 1;
-   }
-   return 0;
+   return import_ffmpeg_decode_frame(sc, flushing);
 }
 
 int FFmpegImportFileHandle::WriteData(streamContext *sc)
 {
-
-   size_t pos = 0;
    // Find the stream index in mScs array
    int streamid = -1;
    for (int i = 0; i < mNumStreams; i++)
@@ -751,21 +729,53 @@ int FFmpegImportFileHandle::WriteData(streamContext *sc)
    }
 
    // Allocate the buffer to store audio.
+   int insamples = sc->m_decodedAudioSamplesValidSiz / sc->m_samplesize;
    int nChannels = sc->m_stream->codec->channels < sc->m_initialchannels ? sc->m_stream->codec->channels : sc->m_initialchannels;
-   int16_t **tmp = (int16_t**)malloc(sizeof(short*)*nChannels);
+   uint8_t **tmp = (uint8_t **) malloc(sizeof(uint8_t *) * nChannels);
    for (int chn = 0; chn < nChannels; chn++)
    {
-      tmp[chn] = (int16_t*)malloc(sizeof(int16_t)*sc->m_decodedAudioSamplesValidSiz/sizeof(int16_t)/sc->m_stream->codec->channels);
+      tmp[chn] = (uint8_t *) malloc(sc->m_osamplesize * (insamples / nChannels));
    }
 
-   // Separate the channels
+   // Separate the channels and convert input sample format to 16-bit
+   uint8_t *in = sc->m_decodedAudioSamples;
    int index = 0;
-   while (pos < sc->m_decodedAudioSamplesValidSiz/sizeof(int16_t))
+   int pos = 0;
+   while (pos < insamples)
    {
-      for (int chn=0; chn < sc->m_stream->codec->channels; chn++)
+      for (int chn = 0; chn < sc->m_stream->codec->channels; chn++)
       {
          if (chn < nChannels)
-            tmp[chn][index] = sc->m_decodedAudioSamples[pos];
+         {
+            switch (sc->m_samplefmt)
+            {
+               case SAMPLE_FMT_U8:
+                  ((int16_t *)tmp[chn])[index] = (int16_t) (*(uint8_t *)in - 0x80) << 8;
+               break;
+               
+               case SAMPLE_FMT_S16:
+                  ((int16_t *)tmp[chn])[index] = (int16_t) *(int16_t *)in;
+               break;
+               
+               case SAMPLE_FMT_S32:
+                  ((float *)tmp[chn])[index] = (float) *(int32_t *)in * (1.0 / (1 << 31));
+               break;
+               
+               case SAMPLE_FMT_FLT:
+                  ((float *)tmp[chn])[index] = (float) *(float *)in;
+               break;
+               
+               case SAMPLE_FMT_DBL:
+                  ((float *)tmp[chn])[index] = (float) *(double *)in;
+               break;
+
+               default:
+                  wxLogError(wxT("Stream %d has unrecognized sample format %d."), streamid, sc->m_samplefmt);
+                  return 1;
+               break;
+            }
+         }
+         in += sc->m_samplesize;
          pos++;
       }
       index++;
@@ -774,7 +784,7 @@ int FFmpegImportFileHandle::WriteData(streamContext *sc)
    // Write audio into WaveTracks
    for (int chn=0; chn < nChannels; chn++)
    {
-      mChannels[streamid][chn]->Append((samplePtr)tmp[chn],int16Sample,index);
+      mChannels[streamid][chn]->Append((samplePtr)tmp[chn],sc->m_osamplefmt,index);
       free(tmp[chn]);
    }
 
@@ -783,7 +793,7 @@ int FFmpegImportFileHandle::WriteData(streamContext *sc)
    // Try to update the progress indicator (and see if user wants to cancel)
    int updateResult = eProgressSuccess;
    // PTS (presentation time) is the proper way of getting current position
-   if (sc->m_pkt.pts != AV_NOPTS_VALUE && mFormatContext->duration != AV_NOPTS_VALUE)
+   if (sc->m_pkt.pts != int64_t(AV_NOPTS_VALUE) && mFormatContext->duration != int64_t(AV_NOPTS_VALUE))
    {
       mProgressPos = sc->m_pkt.pts * sc->m_stream->time_base.num / sc->m_stream->time_base.den;
       mProgressLen = (mFormatContext->duration > 0 ? mFormatContext->duration / AV_TIME_BASE: 1);
@@ -805,39 +815,57 @@ int FFmpegImportFileHandle::WriteData(streamContext *sc)
    return updateResult;
 }
 
-void FFmpegImportFileHandle::WriteMetadata(AVFormatContext *avf,Tags *tags)
+void FFmpegImportFileHandle::WriteMetadata(Tags *tags)
 {
    tags->Clear();
 
-   // We are assuming that tags are in UTF8.
-   // TODO: for some formats tags are not in UTF8. Detect and handle that.
-   tags->SetTag(TAG_TITLE,wxString::FromUTF8(avf->title));
-   tags->SetTag(TAG_ARTIST,wxString::FromUTF8(avf->author));
-   //tags->SetTag(TAG_COPYRIGHT,avf->copyright);
-   tags->SetTag(TAG_COMMENTS,wxString::FromUTF8(avf->comment));
-   tags->SetTag(TAG_ALBUM,wxString::FromUTF8(avf->album));
-   tags->SetTag(TAG_YEAR,avf->year);
-   tags->SetTag(TAG_TRACK,avf->track);
-   tags->SetTag(TAG_GENRE,wxString::FromUTF8(avf->genre));
+   GetMetadata(tags, TAG_TITLE, "title");
+   GetMetadata(tags, TAG_ARTIST, "author");
+//   GetMetadata(tags, TAG_COPYRIGHT, "copyright");
+   GetMetadata(tags, TAG_COMMENTS, "comment");
+   GetMetadata(tags, TAG_ALBUM, "album");
+   GetMetadata(tags, TAG_YEAR, "year");
+   GetMetadata(tags, TAG_TRACK, "track");
+   GetMetadata(tags, TAG_GENRE, "genre");
+}
+
+void FFmpegImportFileHandle::GetMetadata(Tags *tags, const wxChar *tag, const char *name)
+{
+   AVMetadataTag *meta;
+
+   meta = av_metadata_get(mFormatContext->metadata, name, NULL, AV_METADATA_IGNORE_SUFFIX);
+   if (meta)
+   {
+      tags->SetTag(tag, wxString::FromUTF8(meta->value));
+   }
 }
 
 
 FFmpegImportFileHandle::~FFmpegImportFileHandle()
 {
+#ifdef EXPERIMENTAL_OD_FFMPEG
+   //ODDecodeFFmpegTask takes ownership and deltes it there.
+   if(!mUsingOD)
+   {
+#endif
    if (FFmpegLibsInst->ValidLibsLoaded())
    {
-      if (mFormatContext) FFmpegLibsInst->av_close_input_file(mFormatContext);
-      FFmpegLibsInst->av_log_set_callback(FFmpegLibsInst->av_log_default_callback);
+      if (mFormatContext) av_close_input_file(mFormatContext);
+      av_log_set_callback(av_log_default_callback);
    }
 
    for (int i = 0; i < mNumStreams; i++)
    {
       if (mScs[i]->m_decodedAudioSamples != NULL)
-         FFmpegLibsInst->av_free(mScs[i]->m_decodedAudioSamples);
+         av_free(mScs[i]->m_decodedAudioSamples);
 
       delete mScs[i];
    }
    free(mScs);
+#ifdef EXPERIMENTAL_OD_FFMPEG
+   }//mUsingOD
+#endif
+
 
    delete mStreamInfo;
 
