@@ -37,6 +37,9 @@
 #include <wx/event.h>
 #include <wx/frame.h>
 #include <wx/intl.h>
+#if defined(__WXMAC__)
+#include <wx/menu.h>
+#endif
 #include <wx/settings.h>
 #include <wx/sizer.h>
 #include <wx/stopwatch.h>
@@ -989,6 +992,7 @@ static const unsigned char beep[] =
 BEGIN_EVENT_TABLE(ProgressDialog, wxDialog)
    EVT_BUTTON(wxID_CANCEL, ProgressDialog::OnCancel)
    EVT_BUTTON(wxID_OK, ProgressDialog::OnStop)
+   EVT_CLOSE(ProgressDialog::OnCloseWindow)
 END_EVENT_TABLE()  
 
 //
@@ -1002,8 +1006,8 @@ ProgressDialog::ProgressDialog(const wxString & title, const wxString & message,
            wxDefaultSize,
            wxDEFAULT_DIALOG_STYLE |
            wxFRAME_FLOAT_ON_PARENT),
-   mDisable(NULL),
-   mLastValue(0)
+   mLastValue(0),
+   mDisable(NULL)
 {
    wxBoxSizer *v;
    wxWindow *w;
@@ -1143,6 +1147,45 @@ ProgressDialog::ProgressDialog(const wxString & title, const wxString & message,
    // delay, we MUST disable other windows/menus anyway since we run the risk
    // of allowing other tasks to run before this one is complete.
    mDisable = new wxWindowDisabler(this);
+
+#if defined(__WXMAC__)
+   // LL:  On the Mac, the parent windows get disabled, but they still respond
+   //      to the close button being clicked and the application quit menu item
+   //      is still enabled.  We do not want the parent window to be destroyed
+   //      while we're active, so we have to kludge around a bit to keep this
+   //      from happening.
+   WindowRef windowRef = (WindowRef) MacGetWindowRef();
+   if (windowRef) {
+      SetWindowModality(windowRef, kWindowModalityAppModal, NULL);
+      BeginAppModalStateForWindow(windowRef);
+   }
+
+   w = wxTheApp->GetTopWindow();
+   if (w) {
+      w = wxGetTopLevelParent(w);
+      if (w) {
+         wxFrame *f = wxStaticCast(w, wxFrame);
+         if (f) {
+            wxMenuBar *bar = f->GetMenuBar();
+            if (bar) {
+               bar->Enable(wxID_PREFERENCES, false);
+               bar->Enable(wxID_EXIT, false);
+            }
+         }
+      }
+   }
+#endif
+
+#if defined(__WXMSW__)
+   // See Bug #334
+   // LL:  On Windows, the application message loop is still active even though
+   //      all of the windows have been disabled.  So, keyboard shortcuts still
+   //      work in windows not related to the progress diawhich allows interaction 
+   //      when it should be blocked.
+   //      This disabled the application message loop so keyboard shortcuts will
+   //      no longer be processed.
+   wxTheApp->SetEvtHandlerEnabled(false);
+#endif
 }
 
 //
@@ -1157,10 +1200,36 @@ ProgressDialog::~ProgressDialog()
       Beep();
    }
 
+#if defined(__WXMSW__)
+   // Undo above fix for bug 334.
+   wxTheApp->SetEvtHandlerEnabled(true);
+#endif
+
+#if defined(__WXMAC__)
+   wxWindow *w = wxTheApp->GetTopWindow();
+   if (w) {
+      w = wxGetTopLevelParent(w);
+      if (w) {
+         wxFrame *f = wxStaticCast(w, wxFrame);
+         if (f) {
+            wxMenuBar *bar = f->GetMenuBar();
+            if (bar) {
+               bar->Enable(wxID_PREFERENCES, true);
+               bar->Enable(wxID_EXIT, true);
+            }
+         }
+      }
+   }
+
+   WindowRef windowRef = (WindowRef) MacGetWindowRef();
+   if (windowRef) {
+      EndAppModalStateForWindow(windowRef);
+   }
+#endif
+
    if (mDisable)
    {
       delete mDisable;
-
    }
 
 #if defined(__WXGTK__)
@@ -1203,6 +1272,21 @@ ProgressDialog::Show(bool show)
       if (!mDisable)
       {
          mDisable = new wxWindowDisabler(this);
+
+         #if defined(__WXMAC__)
+            // LL:  On the Mac, the parent windows get disabled, but they still respond
+            //      to the close button being clicked and the application quit menu item
+            //      is still enabled.  We do not want the parent window to be destroyed
+            //      while we're active, so we have to kludge around a bit to keep this
+            //      from happening.
+            WindowRef windowRef = (WindowRef) MacGetWindowRef();
+            SetWindowModality( windowRef, kWindowModalityAppModal, NULL ) ;
+            BeginAppModalStateForWindow(windowRef);
+
+            wxMenuBar *bar = wxStaticCast(wxGetTopLevelParent(wxTheApp->GetTopWindow()), wxFrame)->GetMenuBar();
+            bar->Enable(wxID_PREFERENCES, false);
+            bar->Enable(wxID_EXIT, false);
+         #endif
       }
    }
 
@@ -1285,7 +1369,14 @@ ProgressDialog::Update(double current, const wxString & message)
 int
 ProgressDialog::Update(wxULongLong_t current, wxULongLong_t total, const wxString & message)
 {
-   return Update((int)(current * 1000 / total), message);
+   if (total != 0)
+   {
+      return Update((int)(current * 1000 / total), message);
+   }
+   else
+   {
+      return Update(1000, message);
+   }
 }
 
 //
@@ -1294,7 +1385,14 @@ ProgressDialog::Update(wxULongLong_t current, wxULongLong_t total, const wxStrin
 int
 ProgressDialog::Update(wxLongLong current, wxLongLong total, const wxString & message)
 {
-   return Update((int)(current.GetValue() * 1000ll / total.GetValue()), message);
+   if (total.GetValue() != 0)
+   {
+      return Update((int)(current.GetValue() * 1000ll / total.GetValue()), message);
+   }
+   else
+   {
+      return Update(1000, message);
+   }
 }
 
 //
@@ -1303,7 +1401,14 @@ ProgressDialog::Update(wxLongLong current, wxLongLong total, const wxString & me
 int
 ProgressDialog::Update(wxLongLong_t current, wxLongLong_t total, const wxString & message)
 {
-   return Update((int)(current * 1000ll / total), message);
+   if (total != 0)
+   {
+      return Update((int)(current * 1000ll / total), message);
+   }
+   else
+   {
+      return Update(1000, message);
+   }
 }
 
 //
@@ -1312,7 +1417,14 @@ ProgressDialog::Update(wxLongLong_t current, wxLongLong_t total, const wxString
 int
 ProgressDialog::Update(int current, int total, const wxString & message)
 {
-   return Update((int)(current *  ((double)(1000.0 / total))), message);
+   if (total != 0)
+   {
+      return Update((int)(current *  ((double)(1000.0 / total))), message);
+   }
+   else
+   {
+      return Update(1000, message);
+   }
 }
 
 //
@@ -1321,7 +1433,14 @@ ProgressDialog::Update(int current, int total, const wxString & message)
 int
 ProgressDialog::Update(double current, double total, const wxString & message)
 {
-   return Update((int)(current * 1000.0 / total), message);
+   if (total != 0)
+   {
+      return Update((int)(current * 1000.0 / total), message);
+   }
+   else
+   {
+      return Update(1000, message);
+   }
 }
 
 //
@@ -1332,7 +1451,13 @@ ProgressDialog::SetMessage(const wxString & message)
 {
    if (!message.IsEmpty())
    {
+      wxSize sizeBefore = this->GetClientSize();
       mMessage->SetLabel(message);
+      wxSize sizeAfter = this->GetBestSize();
+      wxSize sizeNeeded;
+      sizeNeeded.x = wxMax(sizeBefore.x, sizeAfter.x);
+      sizeNeeded.y = wxMax(sizeBefore.y, sizeAfter.y);
+      this->SetClientSize(sizeNeeded);
       wxYieldIfNeeded();
    }
 }
@@ -1352,6 +1477,12 @@ ProgressDialog::OnStop(wxCommandEvent & e)
    mStop = true;
 }
 
+void
+ProgressDialog::OnCloseWindow(wxCloseEvent & WXUNUSED(event))
+{
+   mCancel = true;
+}
+
 void
 ProgressDialog::Beep()
 {
