@@ -129,6 +129,14 @@ in the status bar of Audacity.
 \class TimeTextCtrlAx
 \brief TimeTextCtrlAx gives the TimeTextCtrl Accessibility.
 
+*******************************************************************//**
+
+\class TimeConverter
+\brief TimeConverter has all the time conversion and snapping 
+functionality that used to live in TimeTextCtrl.  The idea is to have 
+a GUI-less class which can do the conversions, so that we can use it 
+in sanpping without having a window created each time.
+
 *//****************************************************************//**
 
 \class BuiltinFormatString
@@ -153,6 +161,7 @@ different formats.
 #include "../Audacity.h"
 #include "../AudacityApp.h"
 #include "TimeTextCtrl.h"
+#include "../Sequence.h"   // for sampleCount
 #include "../Theme.h"
 #include "../AllThemeResources.h"
 #include "../AColor.h"
@@ -252,7 +261,6 @@ TimeTextCtrl::TimeTextCtrl(wxWindow *parent,
                            bool autoPos):
    wxControl(parent, id, pos, size, wxSUNKEN_BORDER | wxWANTS_CHARS),
    mTimeValue(timeValue),
-   mSampleRate(sampleRate),
    mFormatString(formatString),
    mBackgroundBitmap(NULL),
    mDigitFont(NULL),
@@ -261,6 +269,7 @@ TimeTextCtrl::TimeTextCtrl(wxWindow *parent,
    mLastField(1),
    mAutoPos(autoPos)
 {
+   mConverter.mSampleRate = sampleRate;
    /* i18n-hint: Name of time display format that shows time in seconds */
    BuiltinFormatStrings[0].name = _("seconds");
    /* i18n-hint: Format string for displaying time in seconds. Change the comma
@@ -313,7 +322,9 @@ TimeTextCtrl::TimeTextCtrl(wxWindow *parent,
     * unless there aren't 60 seconds in a minute in your locale */
    BuiltinFormatStrings[5].formatStr = _("0100 h 060 m 060 s+.# samples");
    /* i18n-hint: Name of time display format that shows time in samples (at the
-    * current project sample rate) */
+    * current project sample rate).  For example the number of a sample at 1 
+    * second into a recording at 44.1KHz would be 44,100.
+    */
    BuiltinFormatStrings[6].name = _("samples");
    /* i18n-hint: Format string for displaying time in samples (lots of samples).
     * Change the ',' to the 1000s separator for your locale, and translate
@@ -370,7 +381,7 @@ TimeTextCtrl::TimeTextCtrl(wxWindow *parent,
     * for seconds and translate 'frames'. Nice simple time code! */
    BuiltinFormatStrings[12].formatStr = _("0100 h 060 m 060 s+.25 frames");
    /* i18n-hint: Name of time display format that shows time in frames at PAL
-    * TV frame rate (used for European TV */
+    * TV frame rate (used for European TV) */
    BuiltinFormatStrings[13].name = _("PAL frames (25 fps)");
    /* i18n-hint: Format string for displaying time in frames with NTSC frames.
     * Translate 'frames' and leave the rest alone. */
@@ -396,7 +407,7 @@ TimeTextCtrl::TimeTextCtrl(wxWindow *parent,
    mMenuEnabled = true;
    mButtonWidth = 9;
 
-   ParseFormatString();
+   mConverter.ParseFormatString( mFormatString);
    Layout();
    Fit();
    ValueToControls();
@@ -433,7 +444,7 @@ void TimeTextCtrl::UpdateAutoFocus()
 
    mFocusedDigit = 0;
    while (mFocusedDigit < ((int)mDigits.GetCount() - 1)) {
-      wxChar dgt = mValueString[mDigits[mFocusedDigit].pos];
+      wxChar dgt = mConverter.mValueString[mDigits[mFocusedDigit].pos];
       if (dgt != '0') {
          break;
       }
@@ -444,7 +455,7 @@ void TimeTextCtrl::UpdateAutoFocus()
 void TimeTextCtrl::SetFormatString(wxString formatString)
 {
    mFormatString = formatString;
-   ParseFormatString();
+   mConverter.ParseFormatString( mFormatString);
    Layout();
    Fit();
    ValueToControls();
@@ -454,8 +465,8 @@ void TimeTextCtrl::SetFormatString(wxString formatString)
 
 void TimeTextCtrl::SetSampleRate(double sampleRate)
 {
-   mSampleRate = sampleRate;
-   ParseFormatString();
+   mConverter.mSampleRate = sampleRate;
+   mConverter.ParseFormatString( mFormatString);
    Layout();
    Fit();
    ValueToControls();
@@ -469,10 +480,22 @@ void TimeTextCtrl::SetTimeValue(double newTime)
    ControlsToValue();
 }
 
+void TimeTextCtrl::Increment()
+{
+   mFocusedDigit = mDigits.GetCount() - 1;
+   Increase(1);
+}
+
+void TimeTextCtrl::Decrement()
+{
+   mFocusedDigit = mDigits.GetCount() - 1;
+   Decrease(1);
+}
+
 void TimeTextCtrl::EnableMenu(bool enable)
 {
 #if wxUSE_TOOLTIPS
-   wxString tip(_("Use right mouse button or context key to change format"));
+   wxString tip(_("(Use context menu to change format.)"));
    if (enable)
       SetToolTip(tip);
    else {
@@ -536,7 +559,7 @@ wxString TimeTextCtrl::GetBuiltinFormat(const int index)
 
 wxString TimeTextCtrl::GetBuiltinFormat(const wxString &name)
 {
-   int ndx = 1;
+   int ndx = 4; // Default to "hh:mm:ss + milliseconds".
    int i;
 
    for (i=0; i<TimeTextCtrl::GetNumBuiltins(); i++) {
@@ -549,13 +572,24 @@ wxString TimeTextCtrl::GetBuiltinFormat(const wxString &name)
    return TimeTextCtrl::GetBuiltinFormat(ndx);
 }
 
-void TimeTextCtrl::ParseFormatString()
+TimeConverter::TimeConverter()
+{
+   mPrefix = wxT("");
+   mValueTemplate = wxT("");
+   mValueMask = wxT("");
+   mValueString = wxT("");
+
+   mScalingFactor = 1.0f;
+   mSampleRate = 1.0f;
+   mNtscDrop = false;
+}
+
+void TimeConverter::ParseFormatString( const wxString & format)
 {
    mPrefix = wxT("");
    mFields.Clear();
    mScalingFactor = 1.0;
 
-   const wxString format = mFormatString;
    bool inFrac = false;
    int fracMult = 1;
    int numWholeFields = 0;
@@ -697,7 +731,7 @@ void TimeTextCtrl::ParseFormatString()
    }
 }
 
-void TimeTextCtrl::PrintDebugInfo()
+void TimeConverter::PrintDebugInfo()
 {
    unsigned int i;
 
@@ -726,7 +760,7 @@ wxString TimeTextCtrl::GetTimeString()
 {
    ValueToControls();
 
-   return mValueString;
+   return mConverter.mValueString;
 }
 
 bool TimeTextCtrl::Layout()
@@ -783,9 +817,12 @@ bool TimeTextCtrl::Layout()
    pos = 0;
 
    memDC.SetFont(*mLabelFont);
-   memDC.GetTextExtent(mPrefix, &strW, &strH);
+   memDC.GetTextExtent(mConverter.mPrefix, &strW, &strH);
    x += strW;
-   pos += mPrefix.Length();
+   pos += mConverter.mPrefix.Length();
+
+   // Slightly messy trick to save us some prefixing.
+   TimeFieldArray & mFields = mConverter.mFields;
 
    for(i=0; i<mFields.GetCount(); i++) {
       mFields[i].fieldX = x;
@@ -826,7 +863,7 @@ bool TimeTextCtrl::Layout()
 
    memDC.SetTextForeground(*wxBLACK);
    memDC.SetTextBackground(*wxLIGHT_GREY);
-   memDC.DrawText(mPrefix, mBorderLeft, labelTop);
+   memDC.DrawText(mConverter.mPrefix, mBorderLeft, labelTop);
 
    theTheme.SetBrushColour( Brush, clrTimeBack );
    memDC.SetBrush(Brush);
@@ -902,7 +939,7 @@ void TimeTextCtrl::OnPaint(wxPaintEvent &event)
          dc.SetTextBackground(theTheme.Colour( clrTimeBackFocus ));
       }
       int pos = mDigits[i].pos;
-      wxString digit = mValueString.Mid(pos, 1);
+      wxString digit = mConverter.mValueString.Mid(pos, 1);
       int x = box.x + (mDigitBoxW - mDigitW)/2;
       int y = box.y + (mDigitBoxH - mDigitH)/2;
       dc.DrawText(digit, x, y);
@@ -982,15 +1019,18 @@ void TimeTextCtrl::OnMouse(wxMouseEvent &event)
       OnContext(e);
    }
    else if( event.m_wheelRotation != 0 ) {
-      int steps =  event.m_wheelRotation /
-         (event.m_wheelDelta > 0 ? event.m_wheelDelta : 120);
+      double steps = event.m_wheelRotation /
+         (event.m_wheelDelta > 0 ? (double)event.m_wheelDelta : 120.0) + 
+         mScrollRemainder;
+      mScrollRemainder = steps - floor(steps);
+      steps = floor(steps);
 
-      if (steps < 0) {
-         Decrease(-steps);
+      if (steps < 0.0) {
+         Decrease((int)-steps);
          Updated();
       }
       else {
-         Increase(steps);
+         Increase((int)steps);
          Updated();
       }
    }
@@ -1043,6 +1083,12 @@ void TimeTextCtrl::OnCaptureKey(wxCommandEvent &event)
 
 void TimeTextCtrl::OnKeyDown(wxKeyEvent &event)
 {
+   if (mDigits.GetCount() == 0)
+   {
+      mFocusedDigit = 0;
+      return;
+   }
+
    event.Skip(false);
    int keyCode = event.GetKeyCode();
    int digit = mFocusedDigit;
@@ -1056,7 +1102,7 @@ void TimeTextCtrl::OnKeyDown(wxKeyEvent &event)
    if ((keyCode >= WXK_NUMPAD0) && (keyCode <= WXK_NUMPAD9)) keyCode -= WXK_NUMPAD0 - '0';
 
    if (keyCode >= '0' && keyCode <= '9') {
-      mValueString[mDigits[mFocusedDigit].pos] = wxChar(keyCode);
+      mConverter.mValueString[mDigits[mFocusedDigit].pos] = wxChar(keyCode);
       ControlsToValue();
       ValueToControls();
       mFocusedDigit = (mFocusedDigit+1)%(mDigits.GetCount());
@@ -1068,7 +1114,7 @@ void TimeTextCtrl::OnKeyDown(wxKeyEvent &event)
       mFocusedDigit--;
       mFocusedDigit += mDigits.GetCount();
       mFocusedDigit %= mDigits.GetCount();
-      mValueString[mDigits[mFocusedDigit].pos] = '0';
+      mConverter.mValueString[mDigits[mFocusedDigit].pos] = '0';
       ControlsToValue();
       ValueToControls();
       Updated();
@@ -1142,6 +1188,11 @@ void TimeTextCtrl::OnKeyDown(wxKeyEvent &event)
 void TimeTextCtrl::SetFieldFocus(int digit)
 {
 #if wxUSE_ACCESSIBILITY
+   if (mDigits.GetCount() == 0)
+   {
+      mFocusedDigit = 0;
+      return;
+   }
    mFocusedDigit = digit;
    mLastField = mDigits[mFocusedDigit].field + 1;
 
@@ -1180,36 +1231,39 @@ void TimeTextCtrl::Updated()
 
 void TimeTextCtrl::Increase(int steps)
 {
+   // Slightly messy trick to save us some prefixing.
+   TimeFieldArray & mFields = mConverter.mFields;
+
    while(steps > 0) {
       for(unsigned int i=0; i<mFields.GetCount(); i++) {
          if( (mDigits[mFocusedDigit].pos>=mFields[i].pos) && (mDigits[mFocusedDigit].pos<mFields[i].pos+mFields[i].digits)) {   //it's this field
-            if(!mNtscDrop)
+            if(!mConverter.mNtscDrop)
                ControlsToValue();
             else {
-               mNtscDrop = false;
+               mConverter.mNtscDrop = false;
                ControlsToValue();
-               mNtscDrop = true;
+               mConverter.mNtscDrop = true;
             }
-            mTimeValue *= mScalingFactor;
+            mTimeValue *= mConverter.mScalingFactor;
             double mult = pow(10.,mFields[i].digits-(mDigits[mFocusedDigit].pos-mFields[i].pos)-1);
             if (mFields[i].frac)
                mTimeValue += mult/(double)mFields[i].base;
             else
                mTimeValue += mult*(double)mFields[i].base;
-            if(mNtscDrop) {
+            if(mConverter.mNtscDrop) {
                if( (mTimeValue - (int)mTimeValue)*30 < 2 )
                   if( (((int)mTimeValue)%60 == 0) && (((int)mTimeValue)%600 != 0) )
                      mTimeValue = (int)mTimeValue + 2./30.;
             }
             if(mTimeValue<0.)
                mTimeValue=0.;
-            mTimeValue /= mScalingFactor;
-            if(!mNtscDrop)
+            mTimeValue /= mConverter.mScalingFactor;
+            if(!mConverter.mNtscDrop)
                ValueToControls();
             else {
-               mNtscDrop = false;
+               mConverter.mNtscDrop = false;
                ValueToControls();
-               mNtscDrop = true;
+               mConverter.mNtscDrop = true;
                ControlsToValue();
             }
             break;
@@ -1223,36 +1277,39 @@ void TimeTextCtrl::Increase(int steps)
 
 void TimeTextCtrl::Decrease(int steps)
 {
+   // Slightly messy trick to save us some prefixing.
+   TimeFieldArray & mFields = mConverter.mFields;
+
    while(steps > 0) {
       for(unsigned int i=0; i<mFields.GetCount(); i++) {
          if( (mDigits[mFocusedDigit].pos>=mFields[i].pos) && (mDigits[mFocusedDigit].pos<mFields[i].pos+mFields[i].digits)) {   //it's this field
-            if(!mNtscDrop)
+            if(!mConverter.mNtscDrop)
                ControlsToValue();
             else {
-               mNtscDrop = false;
+               mConverter.mNtscDrop = false;
                ControlsToValue();
-               mNtscDrop = true;
+               mConverter.mNtscDrop = true;
             }
-            mTimeValue *= mScalingFactor;
+            mTimeValue *= mConverter.mScalingFactor;
             double mult = pow(10.,mFields[i].digits-(mDigits[mFocusedDigit].pos-mFields[i].pos)-1);
             if (mFields[i].frac)
                mTimeValue -= mult/(double)mFields[i].base;
             else
                mTimeValue -= mult*(double)mFields[i].base;
-            if(mNtscDrop) {
+            if(mConverter.mNtscDrop) {
                if( (mTimeValue - (int)mTimeValue)*30 < 2 )
                   if( (((int)mTimeValue)%60 == 0) && (((int)mTimeValue)%600 != 0) )
                      mTimeValue = (int)mTimeValue - 1./30.;
             }
             if(mTimeValue<0.)
                mTimeValue=0.;
-            mTimeValue /= mScalingFactor;
-            if(!mNtscDrop)
+            mTimeValue /= mConverter.mScalingFactor;
+            if(!mConverter.mNtscDrop)
                ValueToControls();
             else {
-               mNtscDrop = false;
+               mConverter.mNtscDrop = false;
                ValueToControls();
-               mNtscDrop = true;
+               mConverter.mNtscDrop = true;
                ControlsToValue();
             }
             break;
@@ -1266,8 +1323,31 @@ void TimeTextCtrl::Decrease(int steps)
 
 void TimeTextCtrl::ValueToControls()
 {
-   double theValue = mTimeValue * mScalingFactor + .000001;
-   int t_int = int(theValue);
+   mConverter.ValueToControls( mTimeValue  );
+   Refresh(false);
+}
+
+void TimeConverter::ValueToControls( double RawTime )
+{
+   //RawTime = 4.9995f; Only for testing!
+   RawTime = (double)((sampleCount)floor(RawTime * mSampleRate + 0.5)) / mSampleRate; // put on a sample
+   double theValue = RawTime * mScalingFactor + .000001; // what's this .000001 for?
+   int t_int;
+   bool round = true;
+   // We round on the last field.  If we have a fractional field we round using it.
+   // Otherwise we round to nearest integer.
+   for(unsigned int i=0; i<mFields.GetCount(); i++) {
+      if (mFields[i].frac)
+         round = false;
+   }
+   if(round)
+      t_int = int(theValue + 0.5);
+   else
+   {  
+      wxASSERT( mFields[mFields.GetCount()-1].frac );
+      theValue += 0.5f / mFields[mFields.GetCount()-1].base;
+      t_int = int(theValue);
+   }
    double t_frac = (theValue - t_int);
    unsigned int i;
    int tenMins;
@@ -1279,7 +1359,7 @@ void TimeTextCtrl::ValueToControls()
    mValueString = mPrefix;
 
    if(mNtscDrop) {
-      frames = (int)(theValue*30./1.001);
+      frames = (int)(theValue*30./1.001 + 0.5);
       tenMins = frames/17982;
       frames -= tenMins*17982;
       mins = tenMins * 10;
@@ -1307,25 +1387,39 @@ void TimeTextCtrl::ValueToControls()
 
    for(i=0; i<mFields.GetCount(); i++) {
       int value;
+
       if (mFields[i].frac) {
-         value = (int)(t_frac * mFields[i].base);
-         if (mFields[i].range > 0)
-            value = value % mFields[i].range;
+         // JKC: This old code looks bogus to me.
+         // The rounding is not propogating to earlier fields in the frac case.
+         //value = (int)(t_frac * mFields[i].base + 0.5);  // +0.5 as rounding required
+         // I did the rounding earlier.
+         value = (int)(t_frac * mFields[i].base); 
+         // JKC: TODO: Find out what the range is supposed to do.
+         // It looks bogus too.
+         //if (mFields[i].range > 0)
+         //   value = value % mFields[i].range;
       }
       else {
          value = (t_int / mFields[i].base);
          if (mFields[i].range > 0)
             value = value % mFields[i].range;
       }
+
       wxString field = wxString::Format(mFields[i].formatStr, value);
       mValueString += field;
       mValueString += mFields[i].label;
    }
 
-   Refresh(false);
 }
 
+
+
 void TimeTextCtrl::ControlsToValue()
+{
+   mTimeValue = mConverter.ControlsToValue();
+}
+
+double TimeConverter::ControlsToValue()
 {
    unsigned int i;
    double t = 0.0;
@@ -1367,7 +1461,7 @@ void TimeTextCtrl::ControlsToValue()
       t = frames * 1.001 / 30.;
    }
 
-   mTimeValue = t;
+   return t;
 }
 
 #if wxUSE_ACCESSIBILITY
@@ -1481,7 +1575,10 @@ wxAccStatus TimeTextCtrlAx::GetLocation(wxRect & rect, int elementId)
 {
    rect = mCtrl->GetRect();
 
-   if (elementId != wxACC_SELF) {
+   if ((elementId != wxACC_SELF) && 
+         // We subtract 1, below, and need to avoid neg index to mDigits.
+         (elementId > 0)) 
+   {
 //      rect.x += mCtrl->mFields[elementId - 1].fieldX;
 //      rect.width =  mCtrl->mFields[elementId - 1].fieldW;
         rect = mCtrl->mDigits[elementId - 1].digitBox;
@@ -1495,13 +1592,20 @@ wxAccStatus TimeTextCtrlAx::GetLocation(wxRect & rect, int elementId)
 // Gets the name of the specified object.
 wxAccStatus TimeTextCtrlAx::GetName(int childId, wxString *name)
 {
+   // Slightly messy trick to save us some prefixing.
+   TimeFieldArray & mFields = mCtrl->mConverter.mFields;
+      
    wxString value = mCtrl->GetTimeString();
    int field = mCtrl->GetFocusedField();
 
    // Return the entire time string including the control label
    // when the requested child ID is wxACC_SELF.  (Mainly when
    // the control gets the focus.)
-   if (childId == wxACC_SELF) {
+   if ((childId == wxACC_SELF) || 
+         // We subtract 1 from childId in the other cases below, and 
+         // need to avoid neg index to mDigits, so funnel into this clause.
+         (childId < 1)) 
+   {
       *name = mCtrl->GetName();
       if (name->IsEmpty()) {
          *name = mCtrl->GetLabel();
@@ -1513,15 +1617,15 @@ wxAccStatus TimeTextCtrlAx::GetName(int childId, wxString *name)
    // The user has moved from one field of the time to another so
    // report the value of the field and the field's label.
    else if (mLastField != field) {
-      wxString label = mCtrl->mFields[field - 1].label;
-      int cnt = mCtrl->mFields.GetCount();
+      wxString label = mFields[field - 1].label;
+      int cnt = mFields.GetCount();
       wxString decimal = wxLocale::GetInfo(wxLOCALE_DECIMAL_POINT, wxLOCALE_CAT_NUMBER);
 
       // If the new field is the last field, then check it to see if
       // it represents fractions of a second.
       if (field > 1 && field == cnt) {
-         if (mCtrl->mFields[field - 2].label == decimal) {
-            int digits = mCtrl->mFields[field - 1].digits;
+         if (mFields[field - 2].label == decimal) {
+            int digits = mFields[field - 1].digits;
             if (digits == 2) {
                label = _("centiseconds");
             }
@@ -1533,10 +1637,10 @@ wxAccStatus TimeTextCtrlAx::GetName(int childId, wxString *name)
       // If the field following this one represents fractions of a
       // second then use that label instead of the decimal point.
       else if (label == decimal && field == cnt - 1) {
-         label = mCtrl->mFields[field].label;
+         label = mFields[field].label;
       }
 
-      *name = mCtrl->mFields[field - 1].str +
+      *name = mFields[field - 1].str +
               wxT(" ") +
               label +
               wxT(" ") +
@@ -1552,8 +1656,9 @@ wxAccStatus TimeTextCtrlAx::GetName(int childId, wxString *name)
    }
    // The user has updated the value of a field, so report the field's
    // value only.
-   else {
-      *name = mCtrl->mFields[field - 1].str;
+   else if (field > 0)
+   {
+      *name = mFields[field - 1].str;
    }
 
    return wxACC_OK;
